@@ -11,19 +11,18 @@ char utils_failmesg_sccsid[] = "@(#)failmesg.c	1.2 9/4/95 (UKC)";
 
 #include <errno.h>
 #include <stdio.h>
+#include <string.h>
 #include <local/ukcprog.h>
 
 #include "utils.h"
 
-#ifndef __FreeBSD__
-#ifdef __linux__
-extern const char *const sys_errlist[];
-extern int sys_nerr;
-#else
-extern const char* sys_errlist[];
+#if !defined(__STDC__) || (defined(WANT_STRERROR) && !defined(strerror))
+extern char *sys_errlist[];
 extern int sys_nerr;
+#define strerror(n) \
+    (((n) >= 0 && (n) < sys_nerr) ? sys_errlist[n] : "unknown error")
 #endif
-#endif
+
 
 void
 failmesg(mesg, what, path)
@@ -42,11 +41,5 @@ const char *mesg, *what, *path;
 const char *
 get_errno_str()
 {
-	static char buf[40];
-		
-	if (errno > 0 && errno < sys_nerr && *sys_errlist[errno] != '\0')
-		return sys_errlist[errno];
-	
-	sprintf(buf, "[errno=%d (unknown value)]", errno);
-	return buf;
+	return (strerror (errno));
 }
