@@ -7,7 +7,9 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-char utils_filetype_to_string_sccsid[] = "@(#)fttostr.c	1.4 26/4/92 (UKC)";
+char utils_filetype_to_string_sccsid[] = "@(#)fttostr.c	1.5 17 Apr 1994 (UKC)";
+
+#define UKC_WANT_COMMON_UNIX_EXTENSIONS
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -23,21 +25,25 @@ int mode;
 {
 	static char buf[100];
 
-	switch (mode & S_IFMT) {
-		case S_IFDIR:	return "directory";
-		case S_IFCHR:	return "character special file";
-		case S_IFBLK:	return "block special file";
-		case S_IFREG:	return "regular file";
-#ifdef S_IFLNK
-		case S_IFLNK:	return "symbolic link";
+	if (S_ISREG(mode))
+		return "regular file";
+	if (S_ISDIR(mode))
+		return "directory";
+	if (S_ISCHR(mode))
+		return "character special file";
+	if (S_ISBLK(mode))
+		return "block special file";
+	if (S_ISFIFO(mode))
+		return "named pipe";
+#ifdef S_ISSOCK
+	if (S_ISSOCK(mode))
+		return "socket";
 #endif
-#ifdef S_IFSOCK
-		case S_IFSOCK:	return "socket";
+#ifdef S_ISLNK
+	if (S_ISLNK(mode))
+		return "symbolic link";
 #endif
-#ifdef S_IFIFO
-		case S_IFIFO:	return "named pipe";
-#endif
-	}
+	
 	sprintf(buf, "file with unknown mode 0%o", mode);
 	return buf;
 }
