@@ -7,12 +7,19 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-char obj_sel_sccsid[] = "@(#)o_sel.c	1.17 26/7/92 (UKC)";
+char obj_sel_sccsid[] = "@(#)o_sel.c	1.18 21/10/94 (UKC)";
 
+#include <mtrprog/ifdefs.h>
 #include <local/wn.h>
 
 #include <local/ukcprog.h>
 #include <mtrprog/alloc.h>
+#if defined (OS_BSD44) || defined (OS_LINUX)
+#include <sys/time.h>
+static struct timeval timeout = { 0L, 10L };
+#else
+#include <poll.h>
+#endif
 
 #include "obj.h"
 #include "o_priv.h"
@@ -37,6 +44,7 @@ static void uselect PROTO((struct objst *obj, int new_state, int flags));
 static void sel_obj PROTO((struct objst *obj, int new_state, int flags));
 static void get_field_pos PROTO((struct objst *obj, int fnum, int *p_x, int *p_y));
 static struct fdescst *fnum_to_fdesc PROTO((struct objst *obj, int fnum));
+static void update_size PROTO((struct objst *obj));
 
 /*  Stupid Ultrix C compiler can't handle comparison of two pointers
  *  to functions returning void.
@@ -307,15 +315,30 @@ int obj_x, obj_y;
 
 	/*  The edit may have changed the size of a variable size
 	 *  object, so get the size of the object again.
-	 *
-	 *  BUG: what if the height has changed or there are other
-	 *       objects on the same line as this.
 	 */
+	update_size(obj);
+}
+
+void
+obj_update_size(objcode)
+objid_t objcode;
+{
+	update_size(code_to_obj(objcode));
+}
+
+/*  BUG: what if the height has changed or there are other
+ *       objects on the same line as this.
+ */
+static void
+update_size(obj)
+struct objst *obj;
+{
 	if (Odesc[obj->ob_type].od_getsize != NULL) {
 		sz_t size;
 
 		(*Odesc[obj->ob_type].od_getsize)(obj->ob_code,
-						obj->ob_parent->ob_code, &size);
+						  obj->ob_parent->ob_code,
+						  &size);
 		obj->ob_width = size.sz_width;
 		obj->ob_depth = size.sz_depth;
 	}
@@ -884,19 +907,28 @@ int puck_x, puck_y;
 	return (obj != NULL) ? obj->ob_code : NULL;
 }
 
-void
-select_list()
+struct objst *
+select_list(x_threshold, threshold_exceeded)
+     int x_threshold;
+     int *threshold_exceeded;
 {
-	int puck_x, puck_y, obj_x, obj_y, xpos, forwards;
+	int puck_x, puck_y, obj_x, obj_y, xpos, forwards, last_forwards;
 	int orig_x, orig_y, orig_ldepth, below, button, old_inmode;
-	int objwn_height;
+	int objwn_height, scrolled, flipped, old_puck_x, old_puck_y;
 	struct objst *obj, *newobj;
 	struct lptrst *lptr, *newlptr;
-	
+	short orig_type;
+#if !(defined (OS_BSD44) || defined (OS_LINUX))
+	struct pollfd unused;
+#endif
+
+	*threshold_exceeded = 0;
 	obj = NULL;
 	button = wn_getpuck(Obj_wn, &puck_x, &puck_y) & B_ANY;
+	old_puck_x = puck_x;
+	old_puck_y = puck_y;
 	if (!(button & B_ANY))
-		return;
+		return(0);
 	if (button & B_LEFT) {
 		wn_updating_off(Obj_wn);
 		clear_selection();
@@ -904,14 +936,18 @@ select_list()
 	}
 	getobj(puck_x, puck_y, &obj, &obj_x, &obj_y);
 	if (obj == NULL)
-		return;
-
+	{
+	  wn_ungetpuck(Obj_wn, old_puck_x, old_puck_y, B_LEFT);
+	  *threshold_exceeded = -1;
+	  return(0);
+	}
+	orig_type = obj->ob_type;
 	/*  Middle button means select for editing.
 	 */
 	if (button & B_MIDDLE) {
 		wn_ungetpuck(Obj_wn, puck_x, puck_y, B_MIDDLE);
 		select_field(puck_x, puck_y, obj, obj_x, obj_y);
-		return;
+		return(0);
 	}
 
 	lptr = obj->ob_lptr;
@@ -921,7 +957,17 @@ select_list()
 	orig_ldepth = lptr->lp_ldepth;
 	objwn_height = wn_get_height(Obj_wn);
 	old_inmode = wn_inmode(Obj_wn, WN_SAMPLE);
+	scrolled = 0;
+	last_forwards = FALSE;
 	while (wn_getpuck(Obj_wn, &puck_x, &puck_y) & button) {
+	  if (old_puck_x - puck_x > x_threshold ||
+	      old_puck_x - puck_x < -x_threshold)
+	  {
+	    *threshold_exceeded = 1;
+	    wn_ungetpuck(Obj_wn, old_puck_x, old_puck_y, B_LEFT);
+	    break;
+	  }
+		flipped = 0;
 		forwards = FALSE;
 		newlptr = lptr;
 		if (puck_y < obj_y) {
@@ -949,18 +995,58 @@ select_list()
 		}
 		if (obj == newobj || newlptr->lp_ypos < Offs_y ||
 		    newlptr->lp_ypos + newobj->ob_depth >= Offs_y + objwn_height)
-			continue;
+		{
+		  /* RGA add auto scrolling at top and bottom */
+/*		  if(!scrolled)*/
+		  {
+		    if (puck_y> objwn_height)
+		    {
+		      scrolled = 1;
+		      v_scroll(lptr->lp_ldepth);
+		      obj_y -= lptr->lp_ldepth;
+		      orig_y += lptr->lp_ldepth;
+#if defined (OS_BSD44) || defined (OS_LINUX)
+		      select(0, NULL, NULL, NULL, &timeout);
+#else
+		      poll(&unused, 0, 10);
+#endif
+		    }
+		    else
+		      if(puck_y < 0 && Offs_y > 0)
+		      {
+			scrolled = -1;
+			v_scroll(-lptr->lp_ldepth);
+			obj_y += lptr->lp_ldepth;
+			orig_y -= lptr->lp_ldepth;
+#if defined (OS_BSD44) || defined (OS_LINUX)
+		        select(0, NULL, NULL, NULL, &timeout);
+#else
+			poll(&unused, 0, 10);
+#endif
+		      }
+		  }
+		  continue;
+		}
 		/* at this point, newobj points at the destination */
 		wn_updating_off(Obj_wn);
 		do {
 			if (obj_y < orig_y)
-				below = FALSE;
+			{
+			  below = FALSE;
+			  if (scrolled && last_forwards == forwards)
+			    flipped = 1;
+			}
 			else if (obj_y >= orig_y + orig_ldepth)
-				below = TRUE;
+			{
+			  below = TRUE;
+			  if (scrolled && last_forwards == forwards)
+			    flipped = 1;
+			}
 			else
 				below = obj_x >= orig_x;
-			if(below!=forwards&&(obj_x!=orig_x || obj_y!=orig_y))
-				sel_obj(obj, SEL_FLIP, SEL_PUCKSELECT);
+			if (below!=forwards &&
+			   (obj_x!=orig_x || obj_y!=orig_y))
+			  sel_obj(obj, flipped?SEL_ON:SEL_FLIP, SEL_PUCKSELECT);
 			if (forwards) {
 				if (obj->ob_flags & OB_LAST_ON_LINE) {
 					obj_y += lptr->lp_ldepth;
@@ -992,18 +1078,29 @@ select_list()
 				}
 			}
 			if (obj_y < orig_y)
-				below = FALSE;
+			{
+			  below = FALSE;
+			  if (scrolled && last_forwards == forwards)
+			    flipped = 1;
+			}
 			else if (obj_y >= orig_y + orig_ldepth)
-				below = TRUE;
+			{
+			  below = TRUE;
+			  if (scrolled && last_forwards == forwards)
+			    flipped = 1;
+			}
 			else
 				below = obj_x >= orig_x;
 			/* flip whether object is set or not */
 			if(below==forwards&&(obj_x!=orig_x || obj_y!=orig_y))
-				sel_obj(obj, SEL_FLIP, SEL_PUCKSELECT);
+				sel_obj(obj, flipped?SEL_ON:SEL_FLIP, SEL_PUCKSELECT);
 		} while (obj != newobj);
 		wn_updating_on(Obj_wn);
+	  	scrolled = 0;
+		last_forwards = forwards;
 	}
 	(void) wn_inmode(Obj_wn, old_inmode);
+	return (orig_type == obj->ob_type ? obj : 0);
 }
 
 /* ARGSUSED */
@@ -1021,3 +1118,15 @@ objid_t code;
 {
 	return FALSE;
 }
+
+void
+select_object_at(puck_x, puck_y)
+int puck_x, puck_y;
+{
+	struct objst *obj;
+	int junk;
+	
+	getobj(puck_x, puck_y, &obj, &junk, &junk);
+	if (obj != NULL)
+	  sel_obj(obj, SEL_FLIP, SEL_PUCKSELECT);
+}
