@@ -1,13 +1,20 @@
 /*  e_realloc() -- Error checking realloc. */
 
-char ukcprog_realloc_sccsid[] = "@(#)e_realloc.c	1.6 26/4/92 UKC";
+/*  Copyright 1992  Godfrey Paul, University of Kent at Canterbury.
+ *
+ *  You can do what you like with this source code as long as
+ *  you don't try to make money out of it and you include an
+ *  unaltered copy of this message (including the copyright).
+ */
+
+char ukcprog_realloc_sccsid[] = "@(#)e_realloc.c	1.8 30/5/93 UKC";
 
 #ifndef __STDC__
 #include <sys/types.h>	/* for size_t */
 #endif
 
 #include <stdio.h>	/* for NULL */
-#include <ukcstdlib.h>
+#include <stdlib.h>
 
 #include "ukcprog.h"
 
