@@ -69,6 +69,16 @@ typedef char *constvoidptr;
 #define NULL 0
 #endif
 
+/* For systems without 'strerror()'. */
+#ifdef WANT_STRERROR
+#ifndef strerror
+extern char *sys_errlist[];
+extern int sys_nerr;
+#define strerror(n) \
+    (((n) >= 0 && (n) < sys_nerr) ? sys_errlist[n] : "unknown error")
+#endif
+#endif
+
 /*  Macro to concatenate two or three names.  */
 #ifdef CAT
 #undef CAT
