@@ -603,6 +603,14 @@ typedef void (*wn_abort_func_t)wn__PROTO((void));
 typedef void (*wn_deiconise_func_t)wn__PROTO((int wn));
 typedef void (*wn_draw_icon_func_t)wn__PROTO((int wn));
 typedef int (*wn_event_handler_func_t)wn__PROTO((event_t *ev));
+typedef void (*wn_quit_func_t)wn__PROTO((void));
+typedef int (*wn_get_cur_posn_func_t)wn__PROTO((void));
+typedef void (*wn_overlay_insert_string_func_t)wn__PROTO((const char *text,
+							  int pos,
+							  int len,
+							  int prepend_newline,
+							  int fg_color,
+							  int var_coloring_only));
 
 /*  Old name for wn_get_window_handle(), kept for backwards compatibility.
  */
@@ -665,6 +673,12 @@ void wn_draw_line wn__PROTO((int wn, int x1, int y1, int x2, int y2, int colour)
 void wn_invert_line wn__PROTO((int wn, int x1, int y1, int x2, int y2));
 void wn_box_round wn__PROTO((int wn, int x, int y, int width, int height, int colour));
 void wn_invert_box wn__PROTO((int wn, int x, int y, int width, int height));
+void wn_set_cur_posn_func wn__PROTO((wn_get_cur_posn_func_t func));
+void wn_set_overlay_insert_string_func wn__PROTO((wn_overlay_insert_string_func_t func));
+void wn_overlay_insert_string wn__PROTO((const char *text, int pos,
+					 int len, int prepend_newline,
+					 int fg_color,
+					 int var_coloring_only));
 
 /* cursors */
 cursor_t wn_create_cursor wn__PROTO((bitmap_t *cbm, bitmap_t *mask_bm));
@@ -686,6 +700,7 @@ void wn_suggest_resize_inc wn__PROTO((int base_width, int base_height,
 						int width_inc, int height_inc));
 void wn_set_classname wn__PROTO((const char *name));
 const char *wn_open_display wn__PROTO((void));
+void wn_set_quit_func wn__PROTO((wn_quit_func_t func));
 int wn_open_stdwin wn__PROTO((void));
 int wn_create_window wn__PROTO((const char *name));
 int wn_from_wid wn__PROTO((int wid));
