@@ -33,6 +33,7 @@ char wn_wn_init_c_sccsid[] = "@(#)wn_init.c	1.53 25 May 1995 (UKC)";
 #ifdef X11
 #include <X11/Xresource.h>
 #include <X11/Xatom.h>
+#include <X11/Xos.h>
 #endif
 
 #include "wn_bm.h"
@@ -46,6 +47,18 @@ char wn_wn_init_c_sccsid[] = "@(#)wn_init.c	1.53 25 May 1995 (UKC)";
 #include "wn_color.h"
 #include "sccsdata.h"
 
+static void default_quit_func PROTO((void));
+static void default_quit_func() {};
+
+static wn_quit_func_t Quit_func = default_quit_func;
+
+#if !defined(__STDC__) || (defined(WANT_STRERROR) && !defined(strerror))
+extern char *sys_errlist[];
+extern int sys_nerr;
+#define strerror(n) \
+    (((n) >= 0 && (n) < sys_nerr) ? sys_errlist[n] : "unknown error")
+#endif
+
 static int change_env_var PROTO((const char *s));
 
 #ifdef X11
@@ -167,11 +180,7 @@ void process_client_message(event, frame)
   {
     protocol = event->xclient.data.l[0];
     if( protocol == atom_wm_delete_window)
-    {
-      if (target_process_running())
-	target_stop();
-      re_set_exit_event_loop_flag();
-    }
+      (*Quit_func)();
   }
 }
 
@@ -528,18 +537,7 @@ wn_open_stdwin()
 const char *
 _wn_reason()
 {
-#ifndef __FreeBSD__
-#ifdef __linux__
-extern const char *const sys_errlist[];
-extern int sys_nerr;
-#else
-extern const char* sys_errlist[];
-extern int sys_nerr;
-#endif
-#endif
-
-	return (errno > 0 && errno < sys_nerr) ? sys_errlist[errno]
-					       : "unknown reason";
+	return (strerror (errno));
 } 
 
 #ifdef X11
@@ -1364,3 +1362,10 @@ const char *colorname;
 #endif
 }
 
+void
+wn_set_quit_func(func)
+wn_quit_func_t func;
+{
+  Quit_func = func;
+}
+
