@@ -21,6 +21,20 @@ char wn_wn_rop_c_sccsid[] = "@(#)wn_rop.c	1.18 1/3/93 (UKC)";
 #include "wn_rop.h"
 #include "wn_color.h"
 
+static int default_get_cur_posn_func PROTO((void));
+static int default_get_cur_posn_func() {return 0;};
+
+static wn_get_cur_posn_func_t Get_cur_posn_func = default_get_cur_posn_func;
+
+static void default_overlay_insert_string_func PROTO((const char *text, int pos,
+						      int len,
+						      int prepend_newline,
+						      int fg_color,
+						      int var_coloring_only));
+
+static wn_overlay_insert_string_func_t
+Overlay_insert_string_func = default_overlay_insert_string_func;
+
 #ifdef X11
 static Bool get_gexpose_event PROTO((Display *unused_display, XEvent *xev, char *cwin));
 #endif
@@ -360,10 +374,10 @@ int x1,y1,x2,y2,colour;
 
 	  ft_wd = wn_get_sysfont()->ft_width;
 	  ft_ht = wn_get_sysfont()->ft_height;
-	  y_offset = get_cur_posn()%ft_ht;
+	  y_offset = (*Get_cur_posn_func)()%ft_ht;
 	  adjusted_y = (int)(y1 + (int)ft_ht/2)/ft_ht * ft_ht - y_offset;
 	  wn_text_capture(1, 0, &adjusted_y);	/* set last y */
-	  display_area_overlay_insert_string
+	  (*Overlay_insert_string_func)
 	    (dashes, (int)x1/ft_wd, (int)(x2-x1)/ft_wd,
 	     (last_y != adjusted_y), 0, 0);
 	  return;
@@ -479,3 +493,44 @@ int x, y, width, height;
 	wn_invert_line(wn,	x+width, y+height, x,  	      y+height	);
 	wn_invert_line(wn,	x-1,	 y+height, x-1,       y		);
 }
+
+void
+wn_set_cur_posn_func(func)
+wn_get_cur_posn_func_t func;
+{
+  Get_cur_posn_func = func;
+}
+
+static 
+void 
+default_overlay_insert_string_func(text, pos, len, prepend_newline,
+				   fg_color, var_coloring_only)
+const char *text;
+int pos;
+int len;
+int prepend_newline;
+int fg_color;
+int var_coloring_only;
+{
+}
+
+void
+wn_set_overlay_insert_string_func(func)
+wn_overlay_insert_string_func_t func;
+{
+  Overlay_insert_string_func = func;
+}
+
+void
+wn_overlay_insert_string(text, pos, len, prepend_newline,
+			 fg_color, var_coloring_only)
+     const char *text;
+     int pos;
+     int len;
+     int prepend_newline;
+     int fg_color;
+     int var_coloring_only;
+{
+  (*Overlay_insert_string_func)(text, pos, len, prepend_newline,
+				fg_color, var_coloring_only);
+}
