@@ -666,9 +666,11 @@ scan_main_elf_symtab(alloc_pool_t *target_ap, const char *path, int fd,
 	ld = make_libdep(target_ap, path, (Libdep *)NULL, 0);
 	ok = scan_elf_symtab(target_ap, path, fd, ld,
 			       p_mainfunc, p_initfunc, p_solibs);
-	ld->so->symtab->st_modtime = modtime;
 	if (ld->so)
+	{
+	  ld->so->symtab->st_modtime = modtime;
 	  Main_debug_vaddr = ld->so->debug_vaddr;
+	}
 	return ok;
 }
 
@@ -885,7 +887,15 @@ add_solib_entry(alloc_pool_t *ap, symtab_t *st, Elfinfo *el, Solib **p_solibs)
 	}
 
 	if (!get_elf_shlib_info(ap, el, &soname, &rpath, &debug_vaddr))
-	  return FALSE;
+	{
+	  /* RGA completely statically linked targets used to return
+	     FALSE at this point, terminiating the debug seesion.
+	     Changed to continue with null data.
+	     */
+	  soname = 0;
+	  rpath = "";
+	  debug_vaddr = 0;
+	}
 
 	so = alloc(ap, sizeof(Solib));
 	so->symtab = st;
@@ -936,9 +946,10 @@ bool remove_bpts;
 	if (!ip->ip_solibs)
 	  return 0;
 
-	if ((fp = fopen("/tmp/temp-ups.bpts", "w")) == NULL)
+	if ((fp = fopen(get_temp_bpt_filename(), "w")) == NULL)
 	{
-	  errf("Can't create temp saved breakpoints file `/tmp/temp-ups.bpts'");
+	  errf("Can't create temp saved breakpoints file `%s'",
+	       get_temp_bpt_filename());
 	  return 0;
 	}
 	save_all_breakpoints((char *)fp);
