@@ -1294,19 +1294,23 @@ target_t *xp;
 		status.w_T.w_Termsig = SIGKILL;
 		status.w_T.w_Retcode = 0;
 #endif
-/* RGA 6/18/97 moved up to stop double call of unload_shared_library_symtabs() */
-/* when target is killed - wait_for_target() also calls it */
+	}
 
 #ifndef OS_LINUX
 #if defined(OS_SUNOS_4) || defined(OS_BSD44)
+        mark_breakpoints_as_uninstalled(xp);
 	unload_shared_library_symtabs(xp);
 #endif
 #endif
-	
-	}
+	/* RGA commenting out per Ian Edwards <ian@concerto.demon.co.uk> :
+        wait_for_target() only called
+        unload_shared_library_symtabs() if stopres == SR_DIED, on FreeBSD
+        stopres was SR_SIG so it was not called.  Also need to mark
+        breakpoints as uninstalled as they get reloaded.
+
 	else {
 		wait_for_target(xp);
-	}
+	}*/
 	ip->ip_pid = 0;
 }
 
