@@ -133,7 +133,7 @@ get_fi_compiler(fil)
 
   stf_t *stf, *stf_next;
   int symno, min_symno, max_symno;
-  cursor_t old_cursor;
+  /* cursor_t old_cursor; */
   static int checked = 0;
   static Compiler_type sym;
   static char *c_str = NULL;
@@ -168,12 +168,12 @@ get_fi_compiler(fil)
   st = stf->stf_symtab;
   if (!checked)
   {
-    char *c;
+    char *c2;
 
     c_str = (char *)getenv("UPS_COMPILER");
     if (c_str)
-      for (c = c_str; *c; c++)
-	*c = tolower(*c);
+      for (c2 = c_str; *c2; c2++)
+	*c2 = tolower(*c2);
     if (c_str)
     {
       if (!strcmp(c_str, "cc"))
@@ -340,6 +340,8 @@ bool is_header;
 #else
 	symrec.symio = AO_STDATA(fil->fi_symtab)->st_text_symio;
 #endif
+
+	/* This may take a while.  Put up a blocking cursor */
 	
 	fil->fi_flags |= FI_DOING_TYPES;
  	if (!is_header && !(fil->fi_flags & FI_FOUND_COMPILER))	/* RGA */
@@ -522,6 +524,7 @@ bool is_header;
 
 	fil->fi_flags &= ~FI_DOING_TYPES;
 	fil->fi_flags |= FI_DONE_TYPES;
+
 }
 
 static aggr_or_enum_def_t *
@@ -628,12 +631,10 @@ fil_t *fil;
 	var_t *v, *last;
 	taddr_t addr;
 	errf_ofunc_t oldf;
+        cursor_t old_cursor;
 
 	if (fil->fi_flags & FI_DONE_VARS)
 		return fil->fi_block->bl_vars;
-	
-	oldf = errf_set_ofunc(display_message);
-	errf("\bReading symbols of `%s'... ", fil->fi_name_only);
 
 #ifdef AO_ELF
 	if (!elf_scan_then_setup_symio(fil, &symrec.symio))
@@ -641,6 +642,13 @@ fil_t *fil;
 #else
 	symrec.symio = AO_STDATA(fil->fi_symtab)->st_text_symio;
 #endif
+	
+	oldf = errf_set_ofunc(display_message);
+	errf("\bReading symbols of `%s'... ", fil->fi_name_only);
+	/* This may take a while.  Put up a blocking cursor */
+	old_cursor = wn_get_window_cursor(WN_STDWIN);
+	set_bm_cursor(WN_STDWIN, CU_WAIT);
+	indicate_target_menu_search(1);
 
 	stf = AO_FIDATA(fil);
 
@@ -695,7 +703,7 @@ fil_t *fil;
 		while(*s==';')
 		  s++;
 		
-		if(*s == 0 || *s  && *s==' ')
+		if(*s == 0 || *s==' ')
 		  continue;
 
 		parse_name(&symrec, &s, (alloc_pool_t *)NULL, 1,
@@ -721,7 +729,16 @@ fil_t *fil;
 
 		type = Class(stf, &symrec, &s, &class);
 
-		if (nm.n_value != 0)
+		/* RGA: from Russell Browne:
+		   When compiled with SC4 as a C rather than C++ program, UPS
+		   gets a bogus value for static globals out of the symbol table.
+		   Is not a problem with other compilers.
+		   */
+
+		if (nm.n_value != 0 &&
+		    !(stf->stf_compiler_type == CT_CC &&
+		      (class == CL_EXT || class == CL_STAT) &&
+		      fil->fi_language == LANG_C))
 			addr = (taddr_t)nm.n_value + ast->st_base_address;
 		else
 			addr = lookup_global_addr(st, sn->sn_symtab_name);
@@ -743,6 +760,10 @@ fil_t *fil;
 	fil->fi_flags |= FI_DONE_VARS;
 	errf("\bReading symbols of `%s'... done", fil->fi_name_only);
 	errf_set_ofunc(oldf);
+	if (target_menu_search_disabled(0, 0) == FALSE)
+	  indicate_target_menu_search(0);
+	wn_define_cursor(WN_STDWIN, old_cursor);
+
 	return fil->fi_block->bl_vars;
 }
 
@@ -1359,38 +1380,47 @@ func_t *f;
 			addr = nm.n_value;
 #ifdef AO_ELF
 			if (static_addrs_relative(stf->stf_compiler_type)) {
-				int ar;
-				snlist_t *sn;
+			  int ar;
+			  snlist_t *sn;
 				
-				switch (nm.n_type) {
-				case N_STSYM:
-					ar = AR_DATA;
-					break;
-				case N_LCSYM:
-					ar = AR_BSS;
-					break;
-				case N_ROSYM:
-					ar = AR_RODATA;
-					break;
-				default:
-					ar = -1;
-					break;
-				}
+			  switch (nm.n_type) {
+			  case N_STSYM:
+			    ar = AR_DATA;
+			    break;
+			  case N_LCSYM:
+			    ar = AR_BSS;
+			    break;
+			  case N_ROSYM:
+			    ar = AR_RODATA;
+			    break;
+			  default:
+			    ar = -1;
+			    break;
+			  }
 
-				if (ar != -1 &&
-				    symtab_name_to_sn(stf->stf_snlist, name,
-						      &sn)) {
-					name = sn->sn_name;
-					if (!addr) /* RGA for SC4 */
-					  addr = lookup_global_addr
-					    (f->fu_fil->fi_symtab, sn->sn_symtab_name);
-					else
-					  addr += stf->stf_range[ar].base;
-				}
+			  if (ar != -1 &&
+			      symtab_name_to_sn
+			      (stf->stf_snlist, name, 
+			       stf->stf_global_prefix, f->fu_name,
+			       class == CL_LSTAT &&
+			       stf->stf_compiler_type == CT_CC &&
+			       f->fu_fil->fi_language == LANG_CC, &sn)) {
+			    name = sn->sn_name;
+			    /* RGA for SC4 */
+			    if (!addr &&
+				stf->stf_compiler_type == CT_CC &&
+				f->fu_fil->fi_language == LANG_CC) 
+			      addr = lookup_global_addr
+				(f->fu_fil->fi_symtab, sn->sn_symtab_name);
+			    else
+			      addr += stf->stf_range[ar].base;
+			  }
 			}
 #endif
 			
-			if (class == CL_LSTAT)
+			if (class == CL_LSTAT &&
+			    !(stf->stf_compiler_type == CT_CC &&
+			      f->fu_fil->fi_language == LANG_CC))
 			  addr += ast->st_base_address;
 
 			v = ci_make_var(ap, name, class, type, addr);
@@ -1455,9 +1485,14 @@ func_t *f;
 			break;
 		}
 	}
+
 	if (level != 0)
-/*		panic("missing RBRAC");*/
-		errf("missing RBRAC"); /* RGA */
+	{
+	  /* RCB: SC4 does this for optimized code.  When this
+	  ** happens any data about local variables tends to
+	  ** be wrong anyway, so throw it out. */
+	  sane_bracs = FALSE;
+	}
 
 	/*  With some compilers, functions with parameters but no local
 	 *  variables don't get an LBRAC symbol.  If this is the case
@@ -1537,134 +1572,143 @@ int *p_start, *p_lim;
 
 lno_t *
 ao_get_fu_lnos(f)
-func_t *f;
+     func_t *f;
 {
-	lno_t dummy_lno, *last;
-	ao_stdata_t *ast;
-	int max_lnum;
-	symio_t *symio;
-	taddr_t min_addr;
-	fil_t *fil, *initial_fil;
-	int symno, symno_start, symno_lim;
-	unsigned sline_offset, save_sline_offset;
-	unsigned short u;
-	Compiler_type compiler;
+  lno_t dummy_lno, *last;
+  lno_t *lno = NULL;
+  ao_stdata_t *ast;
+  int max_lnum;
+  symio_t *symio;
+  taddr_t min_addr;
+  fil_t *fil, *initial_fil;
+  int symno, symno_start, symno_lim;
+  unsigned sline_offset, save_sline_offset;
+  unsigned short u;
+  Compiler_type compiler;
 
-	if (f->fu_flags & FU_DONE_LNOS)
-		return f->fu__lnos;
+  if (f->fu_flags & FU_DONE_LNOS)
+    return f->fu__lnos;
 
-	ast = AO_STDATA(f->fu_symtab);
+  ast = AO_STDATA(f->fu_symtab);
 
 #ifdef AO_ELF
-	if (!elf_scan_then_setup_symio(f->fu_fil, &symio))
-		return NULL;
+  if (!elf_scan_then_setup_symio(f->fu_fil, &symio))
+    return NULL;
 #else
-	symio = ast->st_text_symio;
+  symio = ast->st_text_symio;
 #endif
-	   
-	/*  First grab all the N_SLINE symbols in the function - we possibly
-	 *  discard some of them later.  On the 386i the lnos are not stored
-	 *  with other symbols, so we use a seperate function to get them.
- 	 */
+          
+  /*  First grab all the N_SLINE symbols in the function - we possibly
+   *  discard some of them later.  On the 386i the lnos are not stored
+   *  with other symbols, so we use a seperate function to get them.
+   */
 #ifdef COFF_SUN386
-	f->fu__lnos = read_func_coff_lnos(symio, f, &last);
+  f->fu__lnos = read_func_coff_lnos(symio, f, &last);
 #else
-	last = &dummy_lno;
-	max_lnum = 0;
-	get_lno_symrange(f, &symno_start, &symno_lim);
+  last = &dummy_lno;
+  max_lnum = 0;
+  get_lno_symrange(f, &symno_start, &symno_lim);
 
-	if (st_get_min_bpt_addr(f, &min_addr) != 0)
-		min_addr = f->fu_addr;
+  if (st_get_min_bpt_addr(f, &min_addr) != 0)
+    min_addr = f->fu_addr;
 
-	fil = initial_fil = AO_FSYMDATA(f)->fs_initial_lno_fil;
-	compiler = ao_compiler(fil, FALSE, CT_UNKNOWN);
-	sline_offset = AO_FSYMDATA(f)->fs_initial_sline_offset;
-	save_sline_offset = sline_offset;
+  fil = initial_fil = AO_FSYMDATA(f)->fs_initial_lno_fil;
+  compiler = ao_compiler(fil, FALSE, CT_UNKNOWN);
+  sline_offset = AO_FSYMDATA(f)->fs_initial_sline_offset;
+  save_sline_offset = sline_offset;
 
-	for (symno = symno_start; symno < symno_lim; ++symno) {
-		nlist_t nm;
-		lno_t *lno;
-		taddr_t addr;
-		const char *name;
+  for (symno = symno_start; symno < symno_lim; ++symno) {
+    nlist_t nm;
+    taddr_t addr;
+    const char *name;
 
-		getsym(symio, symno, &nm);
+    getsym(symio, symno, &nm);
 
-		switch (nm.n_type) {
-		case N_SLINE:
-			break;
+    switch (nm.n_type) {
+    case N_SLINE:
+      break;
 
 #ifdef N_XLINE
-		case N_XLINE:
-			sline_offset = nm.n_desc << 16;
-			save_sline_offset = sline_offset;
-			break;
+    case N_XLINE:
+      sline_offset = nm.n_desc << 16;
+      save_sline_offset = sline_offset;
+      break;
 #endif
 
-		case N_SOL:
-			name = symstring(symio, symno);
+    case N_SOL:
+      name = symstring(symio, symno);
 
-			if (name && *name == '.' && *(name+1) == '/') /* RGA */
-			  name += 2;
+      if (name && *name == '.' && *(name+1) == '/') /* RGA */
+	name += 2;
 
-			if (!find_sol_fil(f->fu_symtab->st_sfiles,
-					  f->fu_fil->fi_path_hint, name,
-					  &fil)) {
-/*	RGA			errf("Warning: can't find file entry for %s",*/
-/*				     name);*/
-				fil = f->fu_fil;
-			}
+      if (!find_sol_fil(f->fu_symtab->st_sfiles,
+			f->fu_fil->fi_path_hint, name,
+			&fil)) {
+	/*    RGA                     errf("Warning: can't find file entry for %s",*/
+	/*                                    name);*/
+	fil = f->fu_fil;
+      }
 
-			if (fil == initial_fil)
-			  sline_offset = save_sline_offset;
-			else
-			  sline_offset = 0;
-			/* RGA gcc puts out non zero values here which upset */
-			/* the addr of the first lno */
-			if (compiler == CT_GNU_CC)
-			  nm.n_value = 0;
-			break;
+      if (fil == initial_fil)
+	sline_offset = save_sline_offset;
+      else
+	sline_offset = 0;
+      /* RGA gcc puts out non zero values here which upset */
+      /* the addr of the first lno */
+      if (compiler == CT_GNU_CC)
+	nm.n_value = 0;
+      break;
 
-		default:
-			continue;
-		}
-				     
+    default:
+      continue;
+    }
+                                    
 #ifdef AO_ELF
-		addr = f->fu_addr + nm.n_value;
+    addr = f->fu_addr + nm.n_value;
 #else
-		addr = nm.n_value + ast->st_base_address;
+    addr = nm.n_value + ast->st_base_address;
 #endif
 
-		if (addr < min_addr)
-			continue;
-		
-		/*		if (compiler == CT_GNU_CC && fil != f->fu_fil)*/
-		/*			continue;*/
-		
-		lno = (lno_t *)alloc(f->fu_symtab->st_apool,sizeof(lno_t));
-		lno->ln_fil = fil;
-		/* RGA use unsigned - for files over 32k ! */
-		u = (unsigned)nm.n_desc;
-		lno->ln_num = u | sline_offset;
-		lno->ln_addr = addr;
+    if ( lno && addr < min_addr)
+      continue;
+               
+    /*              if (compiler == CT_GNU_CC && fil != f->fu_fil)*/
+    /*                      continue;*/
+             
+    lno = (lno_t *)alloc(f->fu_symtab->st_apool,sizeof(lno_t));
+    lno->ln_fil = fil;
+    /* RGA use unsigned - for files over 32k ! */
+    u = (unsigned)nm.n_desc;
+    lno->ln_num = u | sline_offset;
+    lno->ln_addr = addr;
 
-		if (lno->ln_num > max_lnum)
-			max_lnum = lno->ln_num;
+    if (lno->ln_num > max_lnum)
+      max_lnum = lno->ln_num;
 
-		last->ln_next = lno;
-		last = lno;
-	}
+    last->ln_next = lno;
+    last = lno;
+  }
 
-	last->ln_next = NULL;
-	f->fu__lnos = dummy_lno.ln_next;
+  last->ln_next = NULL;
+
+  /* Get the first record.  Unless it was the only
+  ** record, skip to the first one after the function prelude
+  */
+  lno = dummy_lno.ln_next;
+  if  (   lno
+          && lno->ln_addr < min_addr
+	  && lno->ln_next )
+    lno = lno->ln_next;
+  f->fu__lnos = lno;
 #endif
-	
+        
 #if !defined(COFF_SUN386) && !defined(AO_ELF)
-	fixup_initial_lnos(f);
+  fixup_initial_lnos(f);
 #endif
 
-	f->fu_max_lnum = max_lnum;
-	f->fu_flags |= FU_DONE_LNOS;
-	return f->fu__lnos;
+  f->fu_max_lnum = max_lnum;
+  f->fu_flags |= FU_DONE_LNOS;
+  return f->fu__lnos;
 }
 #endif /* !ST_TE */
 
