@@ -51,6 +51,12 @@ char ups_ao_symparse_c_sccsid[] = "@(#)ao_symparse.c	1.7 04 Jun 1995 (UKC)";
  *  page dbx(5) and information in the document "SPARCworks 3.0.x Debugger
  *  Interface" (on the SPARCworks CDROM).
  *
+ *  Information on stabs format for gcc was obtained from:
+ *   THE "STABS" DEBUG FORMAT
+ *      Julia Menapace, Jim Kingdon, David MacKenzie
+ *      Copyright (C) 1992, 1993 Free Software Foundation, Inc.
+ *      http://www.cygnus.com/misc/gnu-win32/stabs.html
+ *
  *  The routine names are the same as the non terminal names in the grammer.
  *  Above each routine is a description of the part of the grammar that
  *  the routine parses.  "Comments" in these grammar fragments are
@@ -95,6 +101,18 @@ static void Enum PROTO((stf_t *stf, symtab_t *st, const char **p_s,
 			aggr_or_enum_def_t *ae, Symrec *sr));
 static aggr_or_enum_def_t *Record PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 					 bool is_struct, bool eval, type_t *type));
+static var_t *GnuBaseClass PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+			   bool eval, Compiler_type compiler));
+static void GnuStructMethods PROTO((stf_t *stf, Symrec *sr, const char **p_s));
+static var_t *SC4BaseClass PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+			   bool eval, Compiler_type compiler));
+static void SC4StructMethods PROTO((stf_t *stf, Symrec *sr, const char **p_s));
+static void SkipPastSemi PROTO((stf_t *stf, Symrec *sr, const char **p_s));
+static void SkipToSplChar PROTO((stf_t *stf, Symrec *sr, const char **p_s, const char* chars));
+static var_t* SC4StaticMembers PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+			   var_t* next, Compiler_type compiler));
+static var_t* SC4VectorTable PROTO((stf_t *stf, Symrec *sr, const char **p_s,
+			   var_t* next, Compiler_type compiler));
 static var_t *Field PROTO((stf_t *stf, Symrec *sr, const char **p_s,
 			   int is_struct, var_t *next, bool eval,
 			   Compiler_type compiler, bool *terminate));
@@ -463,7 +481,7 @@ int min, max;
 	s = symstring(symio, symrec.symno);
 	s2 = 0;
 	len = strlen(s);
-	if (ft->u.defpos.offset > len)
+	if (ft->u.defpos.offset >= len) /* RCB: changed > to >= */
 	{
 	  end = (char *)s + len - 1;
 	  /* RGA SC4.2 folds lines at 400.  */
@@ -1688,6 +1706,7 @@ type_t *type;
 	var_t *members, *last_members;
 	Compiler_type compiler = CT_CC;
 	bool terminate = FALSE;
+	bool sc4 =FALSE;
 	
 	if (stf->stf_compiler_type == CT_UNKNOWN)
 	  compiler = ao_compiler(NULL, FALSE, CT_UNKNOWN);
@@ -1699,12 +1718,22 @@ type_t *type;
 				      (const char *)NULL, typecode, type);
 	ae->ae_is_complete = AE_COMPLETE;
 	ae->ae_size = parse_num(stf, sr, &s);
-	last_members = members = NULL;
+
+	/* Get Base class info */
 	if (is_struct == 2 || *(*p_s - 2) == 'Y') /* RGA SC3 class or struct */
 	{
+	  sc4 = TRUE;
 	  field_scheck(sr, &s, ';');
-	  field_scheck(sr, &s, ';');
+          members = SC4BaseClass(stf,sr,&s, eval, compiler);
+	} else if ( *s == '!')
+	{
+	  bump_str(sr, &s);
+          members = GnuBaseClass(stf,sr,&s, eval, compiler);
+	} else
+	{
+	    members = NULL;
 	}
+	last_members = members;
 	for (;;) {
 		if (compiler == CT_GNU_CC && *s == '\\') {
 			s = get_cont_symstring(sr);
@@ -1727,12 +1756,16 @@ type_t *type;
 		if (members)
 		  last_members = members;
 	}
+	bump_str(sr, &s);
+	if ( sc4)
+	{
+	    SC4StructMethods(stf, sr, &s);
+	    last_members = SC4StaticMembers(stf, sr, &s, last_members, compiler);
+	    SkipPastSemi(stf, sr, &s); /* Skip list of friends */
+	    last_members = SC4VectorTable(stf, sr, &s, last_members, compiler);
+	}
 	ae->ae_aggr_members = last_members;
 
-/* RGA	if (*s != '\0')
-		scheck(sr, &s, ';');*/
-	bump_str(sr, &s);
-        
         if (stf->stf_language == LANG_F90)
                 fix_f90_pointers(stf->stf_symtab->st_apool, ae);
         
@@ -1768,6 +1801,421 @@ aggr_or_enum_def_t *ae;
                 nextaddr = v->va_addr;
         }
 }
+/*
+**   Special parse routines for G++
+**     Added by Russ Browne (russ@amc.com)
+*/
+static var_t*
+GnuBaseClass(stf,sr, p_s, eval, compiler)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+bool eval;
+Compiler_type compiler;
+{
+    /*
+    **    Parse C++ base class as output by g++ -g
+    **
+    **    The input is of the form
+    **
+    **    ! Base_class_count, visibility_digit offset, type_num_of_base;
+    **
+    **	    the visibility and type repeats for each base class.
+    **
+    **     If there are virtual base classes, the offset for these
+    **     is zero, and the base class information is followed by a 
+    **     field of the form $vb<typenum>,typenum,offset
+    **
+    **     Ref: THE "STABS" DEBUG FORMAT
+    **          Section GNU C++ Stabs
+    */
+
+    var_t *v;
+    type_t *rtype;
+    int virtual;
+    int offset;
+
+    var_t *next = NULL;
+    const char* s = *p_s;
+    int nbases = parse_num(stf, sr, &s);
+    int nvirtual = 0;
+
+    scheck(sr,&s, ',');
+    while (nbases--)
+    {
+	virtual =  ( *s++ == '1');
+	s++;	   /* don't care about public/private */
+        offset = parse_num(stf, sr, &s);
+	scheck(sr,&s, ',');
+
+	rtype = TypeId(stf, sr, &s, eval, 0);
+	if ( *s == '=' )
+	{
+	    abort();
+	    while ( *s && *s != ';')
+		s++;
+	}
+	scheck(sr,&s, ';');
+	if ( !virtual)
+	{
+	    v = (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
+	    v->va_addr = offset/8;
+	    v->va_name = NULL;
+	    v->va_language = stf->stf_language;
+	    v->va_flags = VA_BASECLASS;
+	    v->va_class = CL_MOS;
+	    v->va_type = rtype;
+	    v->va_next = next;
+	    ci_make_baseclass_name(v);
+	    next = v;
+	} else 
+	    nvirtual++;
+    }
+    /*
+    **  Now parse the virtual base information
+    */
+    while ( nvirtual-- && (*s == '$' || *s == '.') && !strncmp(s+1, "vb",2))
+    {
+	    field_scheck(sr, &s, ':'); /* Skip over name */
+	    rtype = TypeId(stf, sr, &s, eval, 0);
+	    scheck(sr,&s, ',');
+            offset = parse_num(stf, sr, &s);
+
+	    v = (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
+	    v->va_addr = offset/8;
+	    v->va_name = NULL;
+	    v->va_language = stf->stf_language;
+	    v->va_flags = VA_BASECLASS|VA_HIDE_PTR;
+	    v->va_class = CL_MOS;
+	    v->va_type = rtype;
+	    v->va_next = next;
+	    ci_make_baseclass_name(v);
+	    next = v;
+            scheck(sr,&s, ';');
+    }
+    *p_s = s;
+    return next;
+}
+static void
+GnuStructMethods(stf,sr, p_s)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+{
+    /*
+    **   The gnu compiler outputs a list of methods for
+    **   the class.  We would need this to support calling
+    **   a method from the inserted code.  For now,
+    **   we only want to find the end of the data so the recursive parse
+    **   of other classes can continue.
+    **
+    **   The format of the data is:
+    **     
+    **        method_name::type_def=type_desc return_type(int);
+    **  	:arg_types; protection qualifier virt_static_normal vtblinfo;;"
+    **
+    **    virt_static_normal is indicated by
+    **        .  = normal
+    **        *  = virtual
+    **        ?  = static
+    **
+    **  vtblinfo, if present, has the form
+    **
+    **        vtable_index;class_first_defining;
+    **
+    **
+    **   Our algorithm is to skip to the next ".", '?', or '*'
+    **   then to the next, or third next semicolon, and repeat
+    **   until the next character is also a semicolon.
+    **
+    **    When called out of Field(), the initial method_name:: has
+    **    already been parsed and *p_s points to the type_def.
+    **
+    **     Ref: THE "STABS" DEBUG FORMAT
+    **          Section GNU C++ Stabs
+    */
+    const char* s = *p_s;
+    do
+    {
+	/* Skip to the :' preceeding the arg_types */
+	SkipToSplChar(stf, sr, &s, ":");
+	if (s[0] == ':' && s[1] == ':')
+	{
+	   s+=2;
+	   SkipToSplChar(stf, sr, &s, ":");
+	}
+
+	/* Skip to the . ? or * */
+	SkipToSplChar(stf, sr, &s, ".?*");
+
+	/* If it was a *, skip past the vtbl index in type number of the
+	** class first defining this method 
+	*/
+	if ( *s == '*')
+	{
+	    SkipPastSemi(stf, sr, &s);
+	    SkipPastSemi(stf, sr, &s);
+	}
+
+	/* Next ';' marks end of the method */
+	SkipPastSemi(stf, sr, &s);
+
+	/* Repeat until remaining char is also a ';' */
+    }
+    while (*s && *s != ';');
+
+    /* The gnu type discription may end with ~%typeNum,
+    ** giving the type number of the root type.  We want to
+    ** skip that too.
+    */
+    if ( s[1] == '~'  && s[2] == '%')
+    {
+	s++;
+	SkipToSplChar(stf, sr, &s, ";");
+    }
+    *p_s = s;
+}
+
+/*
+**   Special parse routines for SC4
+**     Added by Russ Browne (russ@amc.com)
+*/
+static var_t*
+SC4BaseClass(stf,sr, p_s, eval, compiler)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+bool eval;
+Compiler_type compiler;
+{
+    /* Parse base class info as output by the SC4 compiler
+    **  Input for each base class is of the form
+    **
+    **       C<offset in bytes>(tnums)
+    **
+    **   For example
+    **
+    **     C0(0,24)C4(0,250;
+    **
+    **   An initial A, B, or C represents a normal base class
+    **   An initial I, J, or K represents a virtual base class
+    **
+    **   Ref: SPARCworks debugger Interface, Chapter 6, Page 60
+    */
+    var_t *next = NULL;
+    var_t *v = NULL;
+    type_t *rtype;
+    const char* s = *p_s;
+    int virtual;
+    int offset;
+    while  (*s != ';')
+    {
+	if ( *s == 'C' || *s=='B' || *s=='A')
+	   virtual = FALSE;
+	else if ( *s == 'K' || *s == 'J' || *s == 'I')
+	   virtual = TRUE;
+	else
+	{
+	    field_scheck(sr, &s, ';');
+	    break;
+	}
+	bump_str(sr, &s);
+	offset = parse_num(stf, sr, &s);
+	rtype = TypeId(stf, sr, &s, eval, 0);
+	v = (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
+	v->va_addr = offset;
+	v->va_name = NULL;
+	v->va_language = stf->stf_language;
+	v->va_flags = VA_BASECLASS;
+	v->va_class = CL_MOS;
+	v->va_type = rtype;
+	v->va_next = next;
+	next = v;
+	if ( virtual )
+	{
+	     type_t *btype = (type_t*)alloc(stf->stf_symtab->st_apool, sizeof(type_t));
+	     btype->ty_code = DT_PTR_TO;
+	     btype->ty_qualifiers = 0;
+	     btype->ty_size = -1;
+	     btype->ty_base = rtype;
+	     v->va_type = btype;
+	     v->va_flags |= VA_HIDE_PTR;
+	}
+	ci_make_baseclass_name(v);
+    }
+    *p_s = ++s;
+    return v;
+}
+static void
+SC4StructMethods(stf,sr, p_s)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+{
+    /*
+    **   The SC4 compiler outputs a list of methods for
+    **   the class.  We would need this to support calling
+    **   a method from the inserted code.  We might also
+    **   use this to remove the unnatural requirment that looking
+    **   at a static member by clicking on its name only works
+    **   in a non-Static member function.
+    **
+    **   For now, we only want to find the end of the list
+    **
+    **   Ref: SPARCworks debugger Interface, Chapter 6, Page 61
+    */
+    SkipPastSemi(stf,sr, p_s);
+}
+static void
+SkipPastSemi(stf,sr, p_s)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+{
+    /* Skip to the next ';' and consume it */
+    const char* s = *p_s;
+    const char* end = strchr(s, ';');
+    while(*s && end == NULL)
+    {
+	s = get_cont_symstring(sr);
+	end = strchr(s, ';');
+    }
+    if ( end )
+       s = end+1;
+    if (*s == '\\')
+         s = get_cont_symstring(sr);
+    *p_s = s;
+}
+static void
+SkipToSplChar(stf,sr, p_s, chars)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+const char* chars;
+{
+    /* Skip to the next character in the string chars */
+    const char* s = *p_s;
+    const char* end = strpbrk(s, chars);
+    while(*s && end == NULL)
+    {
+	s = get_cont_symstring(sr);
+	end = strpbrk(s, chars);
+    }
+    if ( end )
+       s = end;
+    *p_s = s;
+}
+static var_t*
+SC4StaticMembers(stf,sr, p_s, next, compiler)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+var_t* next;
+Compiler_type compiler;
+{
+    /*
+    **   The SC4 compiler outputs a list of static members for
+    **   the class.  Create a var_t for each of them. Get the names,
+    **   demangle them, put the final component in the va_name field
+    **   and the full name in the va_addr field.  Set the va_type field
+    **   to something safe.
+    **
+    **   Ref: SPARCworks debugger Interface, Chapter 6, Page 61
+    */
+
+    static type_t bogusType;
+    const char* s = *p_s;
+
+    if ( bogusType.ty_size == 0 )
+    {
+	/* First time, set up the bogus type incase anybody looks at it */
+	bogusType.ty_code = TY_INT_ASSUMED;
+	bogusType.ty_size = -1;
+    }
+
+    if (*s == '\\')
+      s = get_cont_symstring(sr);
+    while(*s != ';')
+    {
+	var_t* v = (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
+	const char* name = parse_name(sr, &s, stf->stf_symtab->st_apool,
+				  0, compiler);
+	const char* ename = strrchr(name, ':');
+	if ( ename)
+	   v->va_name = ename+1;
+	else
+	   v->va_name = name;
+	v->va_addr = (taddr_t)name;
+	v->va_language = stf->stf_language;
+	v->va_flags = VA_STATIC_MEMBER;
+	v->va_class = CL_MOS;
+	v->va_type = &bogusType; /* point to something safe */
+	v->va_next = next;
+	next = v;
+	if (*s == '\\')
+	  s = get_cont_symstring(sr);
+	if ( *s == ' ')
+	   bump_str(sr, &s);
+    }
+    bump_str(sr, &s);
+    *p_s = s;
+    return next;
+}
+
+static var_t*
+SC4VectorTable(stf,sr, p_s, next, compiler)
+stf_t *stf;
+Symrec *sr;
+const char **p_s;
+var_t* next;
+Compiler_type compiler;
+{
+    /*
+    **   SC4 vector table info consists of two numbers.  The first
+    **   is an algorithm number for accessing the vector table,
+    **   which we ignore.  The second is offset in bytes of the
+    **   virtual function table pointer in the structure.
+    **
+    **   Ref: SPARCworks debugger Interface, Chapter 6, Page 61
+    */
+    static type_t pVoidType[2];
+    const char* s = *p_s;
+
+    if ( !pVoidType[1].ty_base)
+    {
+       /* First time, create a type_t that says void* */
+       pVoidType[0].ty_code = TY_VOID;
+       pVoidType[0].ty_size = 0;
+       pVoidType[0].ty_name = "void";
+       pVoidType[1].ty_code = DT_PTR_TO;
+       pVoidType[1].ty_size = sizeof(void*);
+       pVoidType[1].ty_qualifiers = 0;
+       pVoidType[1].ty_base = &pVoidType[0];
+    }
+    if (*s == '\\')
+	s = get_cont_symstring(sr);
+    if ( *s != ';')
+    {
+	int method = parse_num(stf, sr, &s);
+
+        var_t *v =  (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
+            v->va_name = "_vptr.";
+            v->va_language = stf->stf_language;
+            v->va_flags = VA_VTBLPTR;
+            v->va_type = &pVoidType[1];
+            bump_str(sr, &s);
+            v->va_addr = parse_num(stf, sr, &s);
+	    v->va_class = CL_MOS;
+	    v->va_next = next;
+	    next = v;
+
+	(void) method;  /* Shut up unused var compiler warning */
+    }
+
+    bump_str(sr, &s);
+    *p_s = s;
+    return next;
+}
 	       
 /* Field:
  *	NAME ':' TypeId ',' BitOffset ',' BitSize ';'
@@ -1786,30 +2234,11 @@ bool *terminate;
 	const char *s;
 	char prior_chr;
 	type_t *type, *type1;
-	int width, offset;
+	int width = 0, offset;
 	var_t *v = NULL;
 
 	s = *p_s;
 
-	if (compiler == CT_GNU_CC && *s == '!')
-	{
-	  char *s1;
-	  s1 = strstr(s, "._type_info_direct_bases");
-	  if (s1 != NULL)
-	  {
-	    s = s1;
-	    field_scheck(sr, &s, ';');
-	  }
-	  else
-	  {
-	    field_scheck(sr, &s, ';');
-	    if (*s != ':' && *s != '?' && !isalpha(*s) && *s != '_' &&
-		*s != '$' && *s != '.' && *s != '!')
-	      field_scheck(sr, &s, ';');
-	    /*	    *terminate = TRUE;*/
-	    /*	    return NULL;*/
-	  }
-	}
 	if ((*s == ':' && *(s+1) != '/'))
 	  bump_str(sr, &s);
 	if (compiler == CT_GNU_CC &&
@@ -1821,7 +2250,7 @@ bool *terminate;
 	  return NULL;
 	}
 
-	if (!(*s == '!' || *s == '$'))
+	/* if (!(*s == '!' || *s == '$')) */
 	{
 	  v = (var_t *) alloc(stf->stf_symtab->st_apool, sizeof(var_t));
 	  v->va_name = parse_name(sr, &s, stf->stf_symtab->st_apool,
@@ -1829,6 +2258,30 @@ bool *terminate;
 	  v->va_language = stf->stf_language;
 	  v->va_flags = 0;
 	  v->va_class = is_struct ? CL_MOS : CL_MOU;
+	  if (compiler == CT_GNU_CC )
+	  {
+	      if  ( *v->va_name == '$' || *v->va_name == '.' )
+	      {
+	          if ( !strncmp(v->va_name+1, "vf", 2))
+	          {
+		      /* "$vf<number>: is a gnu virtual function table.
+		      **  we would like to show these and display the
+		      **  table it points to by name.  This would indicate
+		      **  which sub-class of a base class an object is a]
+		      **  member of.  Gdb does this.
+		      */
+        	      v->va_name = (*v->va_name == '$') ? "_vptr$" : "_vptr.";
+	              v->va_flags =  VA_VTBLPTR;
+		      width = 32;
+	          } else
+	          {
+		     v = NULL;  /* compiler generated symbol. Throw away */
+	          }
+	      }
+	  } else if (!strcmp(v->va_name, "__vptr"))
+	  {
+	       v->va_flags =  VA_VTBLPTR; /* clcc virtual function table */
+	  }
 	}
 	if (*s == '\0')
 	{
@@ -1855,15 +2308,18 @@ bool *terminate;
 	  *p_s = s;
 	}
 
-	if ((*s == ':' && *(s+1) == ':'))
-	  bump_str(sr, &s);
-
 	prior_chr = *s == '\\' ? *(s-1) : *s;
 	if (!field_scheck(sr, &s, ':')
-	    || *s == ':' )  /* RCB: ignore struct_method, $_1::Junk */
+	    || *s == ':' )
 	{
-	  *p_s = s;
-	  return NULL;
+	    if  ( *s == ':')
+	    {
+		 s++;
+		 GnuStructMethods(stf, sr, &s);
+	    }
+	    *p_s = s;
+	    *terminate = TRUE;
+	    return NULL;
 	}
 	if (*s == '/')
 	  s += 2;
@@ -1881,6 +2337,19 @@ bool *terminate;
 	  type1 = TypeId(stf, sr, &s, eval, prior_chr); /* just ignore it */
 	}
 
+	if (*s == ':' )   /*   g++ static member */
+	{
+	    v->va_flags = VA_STATIC_MEMBER;
+	    s++;
+	    v->va_addr = (taddr_t) parse_name(sr, &s, stf->stf_symtab->st_apool,
+				    0, compiler);
+	    v->va_type = type;
+	    (void)field_scheck(sr,&s, ';');
+	    *p_s = s;
+	    v->va_next = next;
+	    return v;
+
+	}
 	if (*s != ',')
 	{
 	  if (type == NULL || !field_scheck(sr, &s, ';'))
@@ -1890,7 +2359,7 @@ bool *terminate;
 	  }
 	  width = typesize(type) * 8;
 	  offset = 0;
-}
+	}
 	else
 	  {
 	    if (!field_scheck(sr, &s, ',') || !isdigit(*s)) 
@@ -1900,7 +2369,7 @@ bool *terminate;
 	    }
 	    
 	    offset = parse_num(stf, sr, &s);
-	    if (!field_scheck(sr, &s, ','))
+	    if (width == 0 && !field_scheck(sr, &s, ','))
 	    {
 	      *p_s = s;
 	      return NULL;
@@ -1921,7 +2390,16 @@ bool *terminate;
 #else
 	    if (compiler == CT_GNU_CC)
 	    {
-	      width = parse_signed_num(stf, sr, &s);
+	      if (width == 0 )
+	          width = parse_signed_num(stf, sr, &s);
+	      if ( width == 0)
+	      {
+		 /* RCB: g++, with the -gstabs option, represents
+		 ** static members as having 0 width
+		 */
+	         v->va_flags = VA_STATIC_MEMBER;
+		 offset = 0; /* ensure that va_addr gets 0 */
+	      } else
 	      if (width < 0)
 		width = typesize(type) * 8;
 	    }
