@@ -1035,7 +1035,7 @@ const char **p_mesg;
 	/*  Push the arg count.
 	 */
 	sp -= 4;
-	if (ps_write_data(xp, sp, (char *)&nargs, sizeof(nargs)) != 0) {
+	if (ps_write_data(ip, sp, (char *)&nargs, sizeof(nargs)) != 0) {
 		*p_mesg = "Can't write the arg count to the stack";
 		return -1;
 	}
@@ -1072,7 +1072,7 @@ const char **p_mesg;
 	/*  Push frame onto the stack.
 	 */
 	sp -= sizeof(frame);
-	if (ps_write_data(xp, sp, (char *)&frame, sizeof(frame)) != 0) {
+	if (ps_write_data(ip, sp, (char *)&frame, sizeof(frame)) != 0) {
 		*p_mesg = "Can't push stack frame";
 		return -1;
 	}
@@ -1080,7 +1080,7 @@ const char **p_mesg;
 #if defined(ARCH_SUN3) || defined(ARCH_SUN386) || defined(ARCH_CLIPPER) || \
     defined(ARCH_BSDI386) || defined(ARCH_LINUX386)
 	sp -= 4;
-	if (ps_write_data(xp, sp, (char *)&retpc, sizeof(retpc)) != 0) {
+	if (ps_write_data(ip, sp, (char *)&retpc, sizeof(retpc)) != 0) {
 		*p_mesg = "Can't push return address";
 		return -1;
 	}
@@ -1107,13 +1107,13 @@ const char **p_mesg;
 	 *  sync the target registers with our idea of their values.
 	 */
 #ifdef ARCH_VAX
-	ps_setreg(xp, UPSREG_AP, ap);
-	ps_setreg(xp, UPSREG_FP, fp);
-	ps_setreg(xp, UPSREG_SP, sp);
+	ps_setreg(ip, UPSREG_AP, ap);
+	ps_setreg(ip, UPSREG_FP, fp);
+	ps_setreg(ip, UPSREG_SP, sp);
 #endif /* ARCH_VAX */
 #if defined(ARCH_SUN3) || defined(ARCH_SUN386) || defined(ARCH_BSDI386) || \
     defined(ARCH_LINUX386)
-	ps_setreg(xp, UPSREG_SP, sp);
+	ps_setreg(ip, UPSREG_SP, sp);
 #endif
 #ifdef ARCH_MIPS
 	/*  The first four integer arguments go in registers, but there
@@ -1123,29 +1123,29 @@ const char **p_mesg;
 	 *       floating point).
 	 */
 	sp -= N_REG_ARGS * 4;
-	if (ps_setreg(xp, MIPS_SP_REGNO, sp) != 0) {
+	if (ps_setreg(ip, MIPS_SP_REGNO, sp) != 0) {
 		*p_mesg = "Can't set stack pointer for target function call";
 		return -1;
 	}
 	
 	for (i = 0; i < nargs && i < N_REG_ARGS; ++i) {
-		if (ps_setreg(xp, 4 + i, realargs[i]) != 0)
+		if (ps_setreg(ip, 4 + i, realargs[i]) != 0)
 			panic("regno write failed in cf");
 	}
 
 	/*  Arrange for the function to return to where we've set
 	 *  the breakpoint.
 	 */
-	if (ps_setreg(xp, MIPS_SAVEDPC_REGNO, retpc) != 0)
+	if (ps_setreg(ip, MIPS_SAVEDPC_REGNO, retpc) != 0)
 		panic("regno write failed in cf");
 #endif
 #ifdef ARCH_CLIPPER
-	if (ps_setreg(xp, UPSREG_SP, sp) != 0) {
+	if (ps_setreg(ip, UPSREG_SP, sp) != 0) {
 		*p_mesg = "Can't set stack pointer for target function call";
 		return -1;
 	}
 	for (i = 0; i < nargs && i < N_REG_ARGS; ++i) {
-		if (ps_setreg(xp, i, realargs[i]) != 0)
+		if (ps_setreg(ip, i, realargs[i]) != 0)
 			panic("regno write failed in cf");
 	}
 #endif
@@ -1196,10 +1196,10 @@ const char **p_mesg;
 	/*  Restore the registers we saved.
 	 */
 #ifdef ARCH_VAX
-	ps_setreg(xp, UPSREG_PC, saved_restart_pc);
-	ps_setreg(xp, UPSREG_SP, saved_sp);
-	ps_setreg(xp, 0, saved_r0);
-	ps_setreg(xp, 1, saved_r1);
+	ps_setreg(ip, UPSREG_PC, saved_restart_pc);
+	ps_setreg(ip, UPSREG_SP, saved_sp);
+	ps_setreg(ip, 0, saved_r0);
+	ps_setreg(ip, 1, saved_r1);
 #endif
 #ifdef OS_SUNOS
 	ps_set_all_regs(ip, origregs);
@@ -1207,21 +1207,21 @@ const char **p_mesg;
 #ifdef ARCH_MIPS
 	if (saveregs) {
 		for (i = 1; i < 16; ++i) {
-			if (ps_setreg(xp, i, regtab[i]) != 0)
+			if (ps_setreg(ip, i, regtab[i]) != 0)
 				panic("can't restore registers in cf");
 		}
 		for (i = 24; i < 32 + 20; ++i) {
-			if (ps_setreg(xp, i, regtab[i]) != 0)
+			if (ps_setreg(ip, i, regtab[i]) != 0)
 				panic("can't restore registers in cf");
 		}
 	}
 #endif
 #ifdef ARCH_CLIPPER
 	for (i = 0; i < N_REG_ARGS; ++i) {
-		if (ps_setreg(xp, i, saved_regs[i]) != 0)
+		if (ps_setreg(ip, i, saved_regs[i]) != 0)
 			panic("can't restore regs in cf");
 	}
-	if (ps_setreg(xp, UPSREG_SP, saved_sp) != 0)
+	if (ps_setreg(ip, UPSREG_SP, saved_sp) != 0)
 		panic("can't restore sp in cf");
 #endif
 #ifdef ARCH_BSDI386
