@@ -7,10 +7,11 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-char ups_ci_decl_c_sccsid[] = "@(#)ci_decl.c	1.19 12/9/92 (UKC)";
+char ups_ci_decl_c_sccsid[] = "@(#)ci_decl.c	1.23 04 Jun 1995 (UKC)";
+
+#include <mtrprog/ifdefs.h>
 
 #include <string.h>
-
 #include <stdlib.h>
 
 #include <local/ukcprog.h>
@@ -39,6 +40,7 @@ static aggr_or_enum_def_t *lookup_enum_member PROTO((aggr_or_enum_def_t *ae,
 								const char *tag));
 static typedef_t *name_to_typedef PROTO((typedef_t *tdlist, const char *name));
 static namedesc_t *make_namedesc PROTO((var_t *v));
+static bool is_decl PROTO((var_t *v));
 
 /*  These globals are active during a given run of yyparse() only.
  *
@@ -46,10 +48,9 @@ static namedesc_t *make_namedesc PROTO((var_t *v));
  */
 block_t *Current_block;
 func_t *Funclist;
-static func_t *Old_funclist;
-static initlist_t *Old_initlist;
 static var_t *Varlist;
 ci_resolve_name_func_t Resolve_name_func;
+static bool Doing_formal_parameters = FALSE;
 
 static bool Make_externs_toplevel;
 
@@ -163,16 +164,20 @@ initlist_t **p_initlist;
 			const char *name;
 			bool is_bitfield;
 			lexinfo_t *lx;
-			type_t *type;
+			type_t *type, *basetype;
 			var_t *v;
 			bool want_decl;
 
 			is_bitfield = dr->dr_type != NULL &&
-						dr->dr_type->ty_code == TY_BITFIELD;
+					   dr->dr_type->ty_code == TY_BITFIELD;
 
-			type = ci_push_types(dn->dn_basetype, dr->dr_type);
 
-			if (type->ty_code == TY_IDENTIFIER) {
+			if ((basetype = ci_get_dn_basetype(dn)) != NULL)
+				type = ci_push_types(basetype, dr->dr_type);
+			else
+				type = NULL;
+
+			if (type != NULL && type->ty_code == TY_IDENTIFIER) {
 				name = type->ty_identifier->id_name;
 				lx = type->ty_identifier->id_lexinfo;
 				type = type->ty_base;
@@ -182,52 +187,43 @@ initlist_t **p_initlist;
 				lx = NULL;
 			}
 
-			if (type == NULL) {
-				if (name != NULL)
-					diagf(ET_WARNING, lx,
-					     "Type defaults to int for `%s'",
-									name);
-				type = ci_code_to_type(TY_INT);
-			}
-
-			if (is_bitfield)
+			if (type != NULL && is_bitfield)
 				type = check_bitfield_type(type, lx);
 			
 			/*  We don't object to void types here if the name
 			 *  is null.  This is so we don't bomb on the
 			 *  special case prototype syntax f(void).
 			 */
-			if (type->ty_code == TY_VOID && name != NULL) {
+			if (type != NULL && type->ty_code == TY_VOID &&
+								name != NULL) {
 				diagf(ET_ERROR, lx,
 					"Illegal type void for `%s'", name);
 				type = ci_code_to_type(TY_INT);
 			}
 								
-			v = ci_make_var(Parse_alloc_id, name, class, type, 0);
+			v = ci_make_var(Parse_apool, name, class, type, 0);
 			v->va_lexinfo = lx;
+			v->va_flags |= VA_IS_CI_VAR;
 
-			/*  If this is just a declaration (class CL_DECL)
-			 *  we may be declaring the type of a non CI variable,
-			 *  so be conservative and only set the CI flag for
-			 *  definitions.
-			 */
-			if (class != CL_DECL)
-				v->va_flags = VA_IS_CI_VAR;
-
-			if (type->ty_code == DT_FUNC_RETURNING &&
-					    (class == CL_EXT || class == CL_AUTO))
+			if (type != NULL &&
+			    type->ty_code == DT_FUNC_RETURNING &&
+			    (class == CL_EXT || class == CL_AUTO))
 				v->va_class = CL_DECL;
 
-			if (dr->dr_initialiser != NULL) {
+			if (type != NULL && dr->dr_initialiser != NULL) {
 				initlist_t *il;
 
-				il = ci_make_var_initialiser(v, dr->dr_initialiser);
+				il = ci_make_var_initialiser(v,
+							dr->dr_initialiser);
 				if (il != NULL) {
 					il->il_next = initlist;
 					initlist = il;
 				}
 			}
 
+			if (type != NULL && type->ty_code == DT_FUNC_RETURNING)
+				ci_rewrite_func_params(type->ty_funcret);
+
 			want_decl = TRUE;
 
 			if (name != NULL) {
@@ -235,7 +231,7 @@ initlist_t **p_initlist;
 
 				name = v->va_name;
 				nd = lookup_name_in_varlist(vlist, name);
-				if (nd != NULL) {
+				if (nd != NULL && type != NULL) {
 					check_redeclaration(nd, v);
 					want_decl = decl_wanted(nd, v, initlist);
 				}
@@ -243,7 +239,7 @@ initlist_t **p_initlist;
 					 default_class == CL_AUTO ||
 					 default_class == CL_TYPEDEF) {
 					nd = lookup_name_in_block(block, name);
-					if (nd != NULL) {
+					if (nd != NULL && type != NULL) {
 						check_redeclaration(nd, v);
 						want_decl = decl_wanted(nd, v,
 									initlist);
@@ -316,6 +312,9 @@ const char *what;
 	for (v = vlist; v != NULL; v = v->va_next) {
 		type = v->va_type;
 
+		if (type == NULL)
+			continue;
+
 		if (type->ty_code == DT_FUNC_RETURNING)
 			type = type->ty_base;
 		else if (v->va_class == CL_DECL)
@@ -405,7 +404,7 @@ enum_member_t *enum_members;
 		 *  the type that don't change when an incomplete type is
 		 *  completed.
 		 */
-		ae = ci_make_aggr_or_enum_def(Parse_alloc_id, tag, typecode,
+		ae = ci_make_aggr_or_enum_def(Parse_apool, tag, typecode,
 								(type_t *)NULL);
 
 		ae->ae_next = Current_block->bl_aggr_or_enum_defs;
@@ -496,7 +495,8 @@ type_t *type;
 {
 	if (type->ty_code == TY_U_STRUCT || type->ty_code == TY_U_UNION)
 		return TRUE;
-	if (type->ty_code == DT_ARRAY_OF && type->ty_dim->di_hdynamic)
+	if (type->ty_code == DT_ARRAY_OF &&
+	    type->ty_dim->di_htype != DT_CONSTANT)
 		return TRUE;
 	return FALSE;
 }
@@ -514,28 +514,25 @@ lexinfo_t *lx;
 	switch (type->ty_base->ty_code) {
 	case TY_INT:
 	case TY_UINT:
-		type->ty_bitfield->bf_code = type->ty_base->ty_code;
 		break;
 	case TY_ULONG:
 	case TY_USHORT:
 	case TY_UCHAR:
 		diagf(ET_WARNING, lx, "Illegal bitfield type %s",
 					ci_basetype_name(type->ty_base));
-		type->ty_bitfield->bf_code = TY_UINT;
 		break;
 	case TY_LONG:
 	case TY_SHORT:
 	case TY_CHAR:
 		diagf(ET_WARNING, lx, "Illegal bitfield type %s",
 					ci_basetype_name(type->ty_base));
-		type->ty_bitfield->bf_code = TY_INT;
 		break;
 	default:
 		diagf(ET_ERROR, lx, "Illegal bitfield type %s",
 					ci_basetype_name(type->ty_base));
-		type->ty_bitfield->bf_code = TY_NOTYPE;
 		break;
 	}
+	type->ty_bitfield->bf_type = type->ty_base;
 	type->ty_base = NULL;
 	return type;
 }
@@ -560,11 +557,28 @@ taddr_t addr;
 		return addr;
 	}
 
+#ifdef IS_BIG_ENDIAN
+	if (prev == NULL || prev->va_type->ty_code != TY_BITFIELD)
+		offset = -1;
+	else
+		offset = prev->va_type->ty_bitfield->bf_offset - bf->bf_width;
+
+	if (bf->bf_width == 0 || offset < 0) {
+		v->va_addr = ci_align_addr_for_type(addr, v->va_type);
+		bf->bf_offset = bits_per_word - bf->bf_width;
+	}
+	else {
+		v->va_addr = prev->va_addr;
+		bf->bf_offset = offset;
+	}
+
+	return v->va_addr + bits_per_word / 8 - bf->bf_offset / 8;
+#else
 	if (prev == NULL || prev->va_type->ty_code != TY_BITFIELD)
 		offset = bits_per_word;
 	else
-		offset = prev->va_type->ty_bitfield->bf_width +
-					prev->va_type->ty_bitfield->bf_offset;
+		offset = prev->va_type->ty_bitfield->bf_offset -
+					prev->va_type->ty_bitfield->bf_width;
 
 	if (bf->bf_width == 0 || offset + bf->bf_width > bits_per_word) {
 		v->va_addr = ci_align_addr_for_type(addr, v->va_type);
@@ -574,7 +588,16 @@ taddr_t addr;
 		v->va_addr = prev->va_addr;
 		bf->bf_offset = offset;
 	}
-	return v->va_addr + ci_typesize(v->va_lexinfo, v->va_type);
+
+	return v->va_addr + (bf->bf_offset + bf->bf_width + 7) / 8;
+#endif /* !IS_BIG_ENDIAN */
+}
+
+void
+ci_set_formal_params_flag(val)
+bool val;
+{
+	Doing_formal_parameters = val;
 }
 
 void
@@ -590,20 +613,20 @@ declaration_t *declaration;
 		return;
 	}
 
-	if (Current_block->bl_parent == Outer_ci_block)
-		default_class = CL_ARG;
+	if (Current_block == Outer_ci_block)
+		default_class = CL_EXT;
 	else
-		default_class = (Current_block == Outer_ci_block) ? CL_EXT : CL_AUTO;
-	ci_declarations_to_varlist(declaration, default_class, &vlist, &initlist);
+		default_class = Doing_formal_parameters ? CL_ARG : CL_AUTO;
 
-	/*  If the parent of the current block is the outer block,
-	 *  vlist refers to old-style formal parameters.  We check
-	 *  these in ci_check_func_decls() so don't check them here.
-	 *  This is partly because we want the incomplete type check
-	 *  to come after the rewrite of `int x[]' to `int *x' that
-	 *  happens for formal parameters.
+	ci_declarations_to_varlist(declaration, default_class,
+							&vlist, &initlist);
+
+	/*  We check function parameters in ci_check_func_decls() so don't'
+	 *  check them here.  This is partly because we want the incomplete
+	 *  type check to come after the rewrite of `int x[]' to `int *x'
+	 *  that happens for formal parameters.
 	 */
-	if (Current_block->bl_parent != Outer_ci_block)
+	if (Doing_formal_parameters)
 		ci_complain_about_incomplete_types(vlist, "Variable");
 
 	Current_block->bl_initlist = ci_push_initlist(initlist,
@@ -676,7 +699,7 @@ func_t *f;
 
 	bl = Outer_ci_block;
 	if ((nd = lookup_name_in_varlist(bl->bl_vars, f->fu_name)) == NULL) {
-		v = ci_make_var(Parse_alloc_id, f->fu_name, CL_DECL, f->fu_type, 0);
+		v = ci_make_var(Parse_apool, f->fu_name, CL_DECL, f->fu_type, 0);
 		v->va_lexinfo = f->fu_lexinfo;
 		v->va_next = bl->bl_vars;
 		bl->bl_vars = v;
@@ -831,8 +854,12 @@ bool add_to_parent;
 {
 	block_t *bl, *lastchild, *child;
 
-	child = ci_make_block(Parse_alloc_id, Current_block);
-	child->bl_start_lnum = Lex_env->le_lnum;
+	child = ci_make_block(Parse_apool, Current_block);
+
+	/*  We add one here because symbol tables number from one upwards
+	 *  rather than from zero.
+	 */
+	child->bl_start_lnum = Lex_env->le_lnum + 1;
 
 	if (add_to_parent) {
 		lastchild = NULL;
@@ -850,7 +877,11 @@ bool add_to_parent;
 void
 ci_end_block()
 {
-	Current_block->bl_end_lnum = Lex_env->le_lnum;
+	/*  We add one here because symbol tables number from one upwards
+	 *  rather than from zero.
+	 */
+	Current_block->bl_end_lnum = Lex_env->le_lnum + 1;
+
 	Current_block = Current_block->bl_parent;
 }
 
@@ -858,36 +889,21 @@ void
 ci_free_parse_id(parse_id)
 parse_id_t parse_id;
 {
-	alloc_free_pool(((parse_res_t *)parse_id)->pr_alloc_id);
+	alloc_free_pool(((parse_res_t *)parse_id)->pr_apool);
 }
 
 void
-ci_start_parse_tree(pr, resolve_name_func, block, flags)
-parse_res_t *pr;
+ci_start_parse_tree(resolve_name_func, block, flags)
 ci_resolve_name_func_t resolve_name_func;
 block_t *block;
 unsigned long flags;
 {
 
-	if (pr == NULL) {
-		Parse_alloc_id = alloc_create_pool();
-		Old_funclist = Funclist = NULL;
-		Current_block = ci_make_block(Parse_alloc_id, block);
-		Varlist = NULL;
-	}
-	else {
-		Parse_alloc_id = pr->pr_alloc_id;
+	Parse_apool = alloc_create_pool();
 
-		Old_funclist = Funclist;
-		Funclist = pr->pr_funcs;
-
-		Current_block = ci_make_block(Parse_alloc_id, block);
-
-		Current_block->bl_initlist = pr->pr_block->bl_initlist;
-		Varlist = pr->pr_block->bl_vars;
-	}
-
-	Old_initlist = Current_block->bl_initlist;
+	Funclist = NULL;
+	Current_block = ci_make_block(Parse_apool, block);
+	Varlist = NULL;
 
 	Outer_ci_block = Current_block;
 	Make_externs_toplevel = (flags & CI_MAKE_EXTERNS_TOPLEVEL) != 0;
@@ -916,70 +932,72 @@ ci_end_parse_tree(parse_succeeded)
 bool parse_succeeded;
 {
 	parse_res_t *pr;
+	fil_t *fil;
+	funclist_t dummy_funclist, *last;
+	initlist_t *il;
+	func_t *f;
+	const char *name;
 
-	if (parse_succeeded) {
-		fil_t *fil;
-		funclist_t *funclist;
-		initlist_t *il;
-		func_t *f;
-
-		if (Current_block != Outer_ci_block)
-			ci_panic("block level botch in ept");
-
-		pr = (parse_res_t *)alloc(Parse_alloc_id, sizeof(parse_res_t));
-		pr->pr_alloc_id = Parse_alloc_id;
-		pr->pr_block = Current_block;
-		pr->pr_funcs = Funclist;
-		pr->pr_block->bl_vars = push_var_definitions(pr->pr_block->bl_vars,
-									Varlist);
-
-		/*  Build a fil structure and point all the functions' fu_fil
-		 *  pointer at it.
-		 */
-
-		fil = NEW(fil_t);
-
-		funclist = NULL;
-		for (f = pr->pr_funcs; f != Old_funclist; f = f->fu_next) {
-			funclist_t *fl;
-
-			f->fu_fil = fil;
-			fl = NEW(funclist_t);
-			fl->fl_func = f;
-			fl->fl_prev = funclist;
-			funclist = fl;
-		}
-
-		for (il = pr->pr_block->bl_initlist; il != Old_initlist;
-								il = il->il_next) {
-			il->il_fil = fil;
-		}
-
-		if (pr->pr_funcs != NULL)
-			fil->fi_name = pr->pr_funcs->fu_lexinfo->lx_filename;
-		else if (pr->pr_block->bl_vars != NULL)
-			fil->fi_name = pr->pr_block->bl_vars->va_lexinfo->lx_filename;
-		else
-			fil->fi_name = "Unknown name";
-
-		fil->fi_path_hint = NULL;
-		fil->fi_language = LANG_C;
-		fil->fi_flags = FI_DONE_VARS;
-		fil->fi_stf = 0;
-		fil->fi_so = NULL;
-		fil->fi_editblocks_id = 0;
-		fil->fi_block = Current_block;
-		fil->fi_funclist_head = NULL;	/* for safety */
-		fil->fi_funclist_tail = funclist;
-		fil->fi_next = NULL;
+	if (!parse_succeeded) {
+		alloc_free_pool(Parse_apool);
+		return NULL;
 	}
-	else {
-		alloc_free_pool(Parse_alloc_id);
-		pr = NULL;
+
+	if (Current_block != Outer_ci_block)
+		ci_panic("block level botch in ept");
+
+	pr = (parse_res_t *)alloc(Parse_apool, sizeof(parse_res_t));
+	pr->pr_apool = Parse_apool;
+	pr->pr_block = Current_block;
+	pr->pr_funcs = Funclist;
+	pr->pr_block->bl_vars = push_var_definitions(pr->pr_block->bl_vars,
+								   Varlist);
+
+	/*  Build a fil structure and point all the functions' fu_fil
+	 *  pointer at it.
+	 */
+
+	if (pr->pr_funcs != NULL)
+		name = pr->pr_funcs->fu_lexinfo->lx_filename;
+	else if (pr->pr_block->bl_vars != NULL)
+		name = pr->pr_block->bl_vars->va_lexinfo->lx_filename;
+	else
+		name = "Unknown name";
+
+	fil = ci_make_fil(Parse_apool, name, (char *)NULL, Current_block,
+			  (fil_t *)NULL);
+
+	fil->fi_flags |= FI_DONE_VARS;
+
+	last = &dummy_funclist;
+	for (f = pr->pr_funcs; f != NULL; f = f->fu_next) {
+		funclist_t *fl;
+
+		f->fu_fil = fil;
+		fl = NEW(funclist_t);
+		fl->fl_func = f;
+
+		last->fl_next = fl;
+		last = fl;
 	}
+	last->fl_next = NULL;
+	fil->fi_funclist = dummy_funclist.fl_next;
+
+	for (il = pr->pr_block->bl_initlist; il != NULL; il = il->il_next)
+		il->il_fil = fil;
+
 	return pr;
 }
 
+static bool
+is_decl(v)
+var_t *v;
+{
+	return v->va_class == CL_DECL ||
+	       (v->va_class == CL_STAT && v->va_type->ty_code ==
+							DT_FUNC_RETURNING);
+}
+
 static void
 check_redeclaration(nd, v)
 namedesc_t *nd;
@@ -989,10 +1007,10 @@ var_t *v;
 
 	var_nametype = (v->va_type->ty_code == DT_FUNC_RETURNING) ? NT_FUNCNAME
 								  : NT_VARNAME;
-
+	
 	if ((nd->nd_nametype == NT_VARNAME || nd->nd_nametype == NT_FUNCNAME) &&
 	    (var_nametype == NT_VARNAME || var_nametype == NT_FUNCNAME) &&
-	    (nd->nd_var->va_class == CL_DECL || v->va_class == CL_DECL)) {
+	    (is_decl(nd->nd_var) || is_decl(v))) {
 		ci_complain_if_types_differ(v->va_name,
 				    nd->nd_var->va_type, nd->nd_var->va_lexinfo,
 				    v->va_type, v->va_lexinfo);
@@ -1018,19 +1036,32 @@ declaration_t *declarations;
 		typedef_t *td;
 		type_t *type;
 
-		/*  We make a copy of the type because we are about to change
-		 *  one of its fields (ty_typedef).
+		/*  We are about to change the ty_typedef field of v->va_type.
+		 *  If the type has a tag (i.e. is a struct, union or enum)
+		 *  then we want this to happen, because it is necessary to
+		 *  make the corrent tie up with structs and unions that are
+		 *  typedef'ed before they are defined.  Otherwise we make
+		 *  a copy.
 		 */
-		type = ci_make_type(Parse_alloc_id, TY_NOTYPE);
-		*type = *v->va_type;
+		switch (v->va_type->ty_code) {
+		case TY_STRUCT:
+		case TY_U_STRUCT:
+		case TY_UNION:
+		case TY_U_UNION:
+		case TY_ENUM:
+		case TY_U_ENUM:
+			type = v->va_type;
+			break;
+		default:
+			type = ci_make_type(Parse_apool, TY_NOTYPE);
+			*type = *v->va_type;
+			break;
+		}
 
-		td = NEW(typedef_t);
-		td->td_name = v->va_name;
+		td = ci_make_typedef(Parse_apool, v->va_name, type);
 		td->td_lexinfo = v->va_lexinfo;
-		td->td_type = type;
-		td->td_type->ty_typedef = td;
+		
 		td->td_next = Current_block->bl_typedefs;
-
 		Current_block->bl_typedefs = td;
 	}
 }
