@@ -7,7 +7,7 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-char ups_ci_lex_c_sccsid[] = "@(#)ci_lex.c	1.16 20/5/92 (UKC)";
+char ups_ci_lex_c_sccsid[] = "@(#)ci_lex.c	1.25 21 Jun 1995 (UKC)";
 
 #define DEBUG
 
@@ -15,6 +15,11 @@ char ups_ci_lex_c_sccsid[] = "@(#)ci_lex.c	1.16 20/5/92 (UKC)";
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <errno.h>
+
+#ifndef __STDC__
+char *memcpy();
+#endif
 
 #include <local/ukcprog.h>
 
@@ -34,6 +39,15 @@ typedef struct message_action_entryst {
 	struct message_action_entryst *ma_next;
 } message_action_entry_t;
 
+#ifndef __STDC__
+extern double strtod();
+extern char *memchr();
+#endif
+
+static bool convert_number PROTO((const char *str, int radix,
+				  char **p_endstr, unsigned long *p_val));
+
+
 #ifdef DEBUG
 static const char *tokname PROTO((token_t token));
 #endif
@@ -52,57 +66,60 @@ static struct {
 	token_t token;
 	bool need_lexinfo;
 } Keytab[] = {
-	"auto",		AUTO,		FALSE,
-	"break",	BREAK,		TRUE,
-	"case",		CASE,		FALSE,
-	"char",		CHAR,		FALSE,
-	"const",	CONST,		FALSE,
-	"continue",	CONTINUE,	TRUE,
-	"default",	DEFAULT,	FALSE,
-	"do",		DO,		FALSE,
-	"double",	DOUBLE,		FALSE,
-	"else",		ELSE,		FALSE,
-	"enum",		ENUM,		FALSE,
-	"extern",	EXTERN,		FALSE,
-	"float",	FLOAT,		FALSE,
-	"for",		FOR,		TRUE,
-	"goto",		GOTO,		FALSE,
-	"if",		IF,		FALSE,
-	"int",		INT,		FALSE,
-	"long",		LONG,		FALSE,
-	"register",	REGISTER,	FALSE,
-	"return",	RETURN,		TRUE,
-	"short",	SHORT,		FALSE,
-	"signed",	SIGNED,		FALSE,
-	"sizeof",	SIZEOF,		FALSE,
-	"static",	STATIC,		FALSE,
-	"struct",	STRUCT,		FALSE,
-	"switch",	SWITCH,		FALSE,
-	"typedef",	TYPEDEF,	FALSE,
-	"union",	UNION,		FALSE,
-	"unsigned",	UNSIGNED,	FALSE,
-	"void",		VOID,		FALSE,
-	"volatile",	VOLATILE,	FALSE,
-	"while",	WHILE,		FALSE,
+	{ "auto",	AUTO,		FALSE	},
+	{ "break",	BREAK,		TRUE	},
+	{ "case",	CASE,		FALSE	},
+	{ "char",	CHAR,		FALSE	},
+	{ "const",	CONST,		FALSE	},
+	{ "continue",	CONTINUE,	TRUE	},
+	{ "default",	DEFAULT,	FALSE	},
+	{ "do",		DO,		FALSE	},
+	{ "double",	DOUBLE,		FALSE	},
+	{ "else",	ELSE,		FALSE	},
+	{ "enum",	ENUM,		FALSE	},
+	{ "extern",	EXTERN,		FALSE	},
+	{ "float",	FLOAT,		FALSE	},
+	{ "for",	FOR,		TRUE	},
+	{ "goto",	GOTO,		FALSE	},
+	{ "if",		IF,		FALSE	},
+	{ "int",	INT,		FALSE	},
+	{ "long",	LONG,		FALSE	},
+	{ "register",	REGISTER,	FALSE	},
+	{ "return",	RETURN,		TRUE	},
+	{ "short",	SHORT,		FALSE	},
+	{ "signed",	SIGNED,		FALSE	},
+	{ "sizeof",	SIZEOF,		FALSE	},
+	{ "static",	STATIC,		FALSE	},
+	{ "struct",	STRUCT,		FALSE	},
+	{ "switch",	SWITCH,		FALSE	},
+	{ "typedef",	TYPEDEF,	FALSE	},
+	{ "union",	UNION,		FALSE	},
+	{ "unsigned",	UNSIGNED,	FALSE	},
+	{ "void",	VOID,		FALSE	},
+	{ "volatile",	VOLATILE,	FALSE	},
+	{ "while",	WHILE,		FALSE	},
 };
 #define NKEYS (sizeof Keytab / sizeof *Keytab)
 
 lex_env_t *Lex_env;
 
+#ifndef OS_SUNOS_4		/* use clib version for SunOS */
 /*  BUG: this should be in a library somewhere.
  */
 char *
 strstr(s, pat)
 const char *s, *pat;
 {
-	int len;
+	int len, s_len, i;
 
 	len = strlen(pat);
-	for (; *s != '\0'; ++s)
+	s_len = strlen(s);	/* RGA don't look beyond string */
+	for (i = 0; i <= s_len - len && *s != '\0'; ++s, ++i)
 		if (*s == *pat && memcmp(s, pat, len) ==  0)
 			return (char *)s; /* UGH */
 	return NULL;
 }
+#endif /* !OS_SUNOS_4 */
 
 void
 yyerror(s)
@@ -132,13 +149,14 @@ errtype_t errtype;
 lexinfo_t *lx;
 const char *mesg;
 {
+	static bool last_mesg_suppressed = FALSE;
 	lex_env_t *le;
 	char *wmesg;
 	bool abort_parse;
 
 	le = (lex_env_t *)arg;
 
-	if (le->le_abort_parse)
+	if (le->le_abort_parse || (errtype == ET_MORE && last_mesg_suppressed))
 		return;
 
 	if (errtype == ET_WARNING || errtype == ET_ERROR) {
@@ -174,7 +192,8 @@ const char *mesg;
 		}
 	}
 
-	if (errtype == ET_IGNORE)
+	last_mesg_suppressed = errtype == ET_IGNORE;
+	if (last_mesg_suppressed)
 		return; 
 
 	if (errtype == ET_WARNING) {
@@ -184,23 +203,19 @@ const char *mesg;
 	else
 		wmesg = NULL;
 
-	if (errtype == ET_MORE)
-		abort_parse = (*le->le_report_func)((char *)NULL, 0, 0, mesg);
+	if (errtype == ET_MORE) {
+		abort_parse = (*le->le_report_func)((lexinfo_t *)NULL, mesg);
+	}
 	else {
-		const char *filename;
-		int lnum, cnum;
+		lexinfo_t cur_lexinfo;
 
-		if (lx != NULL) {
-			filename = lx->lx_filename;
-			lnum = lx->lx_lnum;
-			cnum = lx->lx_cnum;
+		if (lx == NULL) {
+			cur_lexinfo.lx_filename = le->le_filename;
+			cur_lexinfo.lx_lnum = le->le_lnum + 1;
+			cur_lexinfo.lx_cnum = le->le_lptr - le->le_line;
+			lx = &cur_lexinfo;
 		}
-		else {
-			filename = le->le_filename;
-			lnum = le->le_lnum + 1;
-			cnum = le->le_lptr - le->le_line;
-		}
-		abort_parse = (*le->le_report_func)(filename, lnum, cnum, mesg);
+		abort_parse = (*le->le_report_func)(lx, mesg);
 	}
 
 	if (wmesg != NULL)
@@ -232,7 +247,7 @@ lex_env_t *le;
 		for (line += 7; *line != '\0' && isspace(*line); ++line)
 			;
 		diagf(ET_WARNING, (lexinfo_t *)NULL, "#pragma `%.*s' ignored",
-							strlen(line) - 1, line);
+						   (int)strlen(line) - 1, line);
 		return line + strlen(line);
 	}
 
@@ -244,10 +259,10 @@ lex_env_t *le;
 	}
 	if (nitems == 2) {
 		char *buf;
-		int len;
+		size_t len;
 
 		len = strlen(name);
-		buf = alloc(Parse_alloc_id, len + 1);
+		buf = alloc(Parse_apool, len + 1);
 		(void) memcpy(buf, name, len + 1);
 		le->le_filename = buf;
 	}
@@ -362,6 +377,8 @@ const char *line;
 #endif
 	for (;;) {
 		for(;;) {
+			const char *s;
+			
 			while (*line != '\0' && isspace(*line))
 				++line;
 			if (*line != '\0')
@@ -372,9 +389,12 @@ const char *line;
 #ifdef DEBUG
 			read_another_line = TRUE;
 #endif
-			if (*line == '#')
-				line = parse_hash_directive(line + 1, le);
+			for (s = line; isspace(*s); ++s)
+				;
+			if (*s == '#')
+				line = parse_hash_directive(s + 1, le);
 		}
+		
 		if (incomment) {
 			if (line == NULL) {
 				diagf(ET_ERROR, (lexinfo_t *)NULL,
@@ -407,6 +427,82 @@ const char *line;
 	return line;
 }
 
+static bool
+convert_number(str, radix, p_endstr, p_val)
+const char *str;
+int radix;
+char **p_endstr;
+unsigned long *p_val;
+{
+	unsigned long val, maxval;
+	static const char digits[] = "0123456789abcdefghijklmnopqrstuvwxyz";
+	bool failed;
+	const char *startstr, *pos;
+
+	errno = 0;
+	startstr = str;
+
+	if (radix == 0) {
+		if (*str == '0') {
+			++str;
+			if (*str == 'x' || *str == 'X') {
+				++str;
+				radix = 16;
+			}
+			else {
+				radix = 8;
+			}
+		}
+		else {
+			radix = 10;
+		}
+	}
+
+	if (radix < 2 || radix > 36) {
+		diagf(ET_ERROR, (lexinfo_t *)NULL,
+		      "Illegal radix %d - not in range 2..36", radix);
+		return FALSE;
+	}
+	
+	val = 0;
+	failed = FALSE;
+	maxval = ~(unsigned long)0 / radix;
+
+	while ((pos = memchr(digits, tolower(*str), (size_t)radix)) != NULL) {
+		unsigned long newval;
+
+		if (val > maxval)
+			failed = TRUE;
+			
+		newval = val * radix + (pos - digits);
+
+		if (newval < val)
+			failed = TRUE; /* wrapped */
+		val = newval;
+		
+		++str;
+	}
+
+	if (isalnum(*str) && *str != 'e' && *str != 'E') {
+		diagf(ET_ERROR, (lexinfo_t *)NULL,
+		      "Illegal digit `%c' in integer constant `%.*s'",
+		      *str, (str + 1 - startstr), startstr);
+		return FALSE;
+	}
+	
+	*p_endstr = (char *)str;
+
+	if (failed) {
+		diagf(ET_ERROR, (lexinfo_t *)NULL,
+		      "Integer constant `%.*s' out of range",
+		      str - startstr, startstr);
+		return FALSE;
+	}
+
+	*p_val = val;
+	return TRUE;
+}
+
 token_t
 yylex()
 {
@@ -454,7 +550,8 @@ yylex()
 			const char *s;
 			char *buf;
 			identifier_t *id;
-			int len, i;
+			size_t len;
+			int i;
 
 			--line;
 			for (s = line; isalnum(*s) || *s == '_' || *s == '$'; ++s)
@@ -475,7 +572,7 @@ yylex()
 				break;
 			}
 					
-			buf = alloc(Parse_alloc_id, len + 1);
+			buf = alloc(Parse_apool, len + 1);
 			(void) memcpy(buf, line, len);
 			buf[len] = '\0';
 
@@ -495,14 +592,12 @@ yylex()
 	case '5': case '6': case '7': case '8': case '9':
 		{
 			char *end;
-			long val;
+			unsigned long val;
 			
-			val = strtol(line - 1, &end, 0);
-			if (end == line - 1) {
-				le->le_lptr = line;
-				diagf(ET_ERROR, (lexinfo_t *)NULL,
-					"Badly formed integer constant \"%s\"",
-					line - 1);
+			/*  BUG: handle signed/unsigned and types correctly.
+			 */
+
+			if (!convert_number(line - 1, 0, &end, &val)) {
 				token = BADTOK;
 			}
 			else if (*end == 'e' || *end == 'E' || *end == '.') {
@@ -730,8 +825,8 @@ string_const_t *sc;
 	static const char badalloc[] =
 				"Unable to allocate memory for string constant";
 	static char *buf;
-	static int bufsize = 0;
-	int opos;
+	static size_t bufsize = 0;
+	size_t opos;
 	bool ok;
 
 	if (bufsize == 0) {
@@ -794,7 +889,7 @@ string_const_t *sc;
 	if (!ok)
 		return BADTOK;
 
-	sc->sc_val = memcpy(allocstr(Parse_alloc_id, opos), buf, opos);
+	sc->sc_val = memcpy(allocstr(Parse_apool, opos), buf, opos);
 	sc->sc_size = opos;
 	return STRING_CONSTANT;
 }
@@ -837,92 +932,92 @@ token_t token;
 		const char *name;
 		token_t token;
 	} tab[] = {
-		"IF",                     IF,
-		"ELSE",                   ELSE,
-		"WHILE",                  WHILE,
-		"FOR",                    FOR,
-		"DO",                     DO,
-		"GOTO",                   GOTO,
-		"BREAK",                  BREAK,
-		"CONTINUE",               CONTINUE,
-		"RETURN",                 RETURN,
-		"SWITCH",                 SWITCH,
-		"CASE",                   CASE,
-		"DEFAULT",                DEFAULT,
-		"SIZEOF",                 SIZEOF,
-		"AUTO",                   AUTO,
-		"REGISTER",               REGISTER,
-		"STATIC",                 STATIC,
-		"EXTERN",                 EXTERN,
-		"TYPEDEF",                TYPEDEF,
-		"VOID",                   VOID,
-		"CHAR",                   CHAR,
-		"SHORT",                  SHORT,
-		"INT",                    INT,
-		"LONG",                   LONG,
-		"FLOAT",                  FLOAT,
-		"DOUBLE",                 DOUBLE,
-		"SIGNED",                 SIGNED,
-		"UNSIGNED",               UNSIGNED,
-		"CONST",                  CONST,
-		"VOLATILE",               VOLATILE,
-		"STRUCT",                 STRUCT,
-		"UNION",                  UNION,
-		"ENUM",                   ENUM,
-		"AND",                    AND,
-		"TILDE",                  TILDE,
-		"NOT",                    NOT,
-		"LESSTHAN",               LESSTHAN,
-		"GREATERTHAN",            GREATERTHAN,
-		"XOR",                    XOR,
-		"OR",                     OR,
-		"PLUS",                   PLUS,
-		"MINUS",                  MINUS,
-		"SLASH",                  SLASH,
-		"PERCENT",                PERCENT,
-		"STAR",                   STAR,
-		"DOT",                    DOT,
-		"COLON",                  COLON,
-		"QUERY",                  QUERY,
-		"SEMI",                   SEMI,
-		"COMMA",                  COMMA,
-		"LPAREN",                 LPAREN,
-		"RPAREN",                 RPAREN,
-		"LBRACE",                 LBRACE,
-		"RBRACE",                 RBRACE,
-		"LBRAC",                  LBRAC,
-		"RBRAC",                  RBRAC,
-		"EQUALS",                 EQUALS,
-		"STAR_EQUALS",            STAR_EQUALS,
-		"SLASH_EQUALS",           SLASH_EQUALS,
-		"PERCENT_EQUALS",         PERCENT_EQUALS,
-		"PLUS_EQUALS",            PLUS_EQUALS,
-		"MINUS_EQUALS",           MINUS_EQUALS,
-		"LSHIFT_EQUALS",          LSHIFT_EQUALS,
-		"RSHIFT_EQUALS",          RSHIFT_EQUALS,
-		"AND_EQUALS",             AND_EQUALS,
-		"XOR_EQUALS",             XOR_EQUALS,
-		"OR_EQUALS",              OR_EQUALS,
-		"ANDAND",                 ANDAND,
-		"OROR",                   OROR,
-		"EQEQ",                   EQEQ,
-		"NOTEQ",                  NOTEQ,
-		"GTEQ",                   GTEQ,
-		"LESSEQ",                 LESSEQ,
-		"LSHIFT",                 LSHIFT,
-		"RSHIFT",                 RSHIFT,
-		"PLUSPLUS",               PLUSPLUS,
-		"MINUSMINUS",             MINUSMINUS,
-		"ARROW",                  ARROW,
-		"ELLIPSIS",               ELLIPSIS,
-		"STRING_CONSTANT",        STRING_CONSTANT,
-		"INTEGER_CONSTANT",       INTEGER_CONSTANT,
-		"CHARACTER_CONSTANT",     CHARACTER_CONSTANT,
-		"FLOATING_CONSTANT",      FLOATING_CONSTANT,
-		"IDENTIFIER",             IDENTIFIER,
-		"TYPEDEF_NAME",           TYPEDEF_NAME,
-		"BADTOK",                 BADTOK,
-		"EOF",                    0,
+		{ "IF",                     IF			},
+		{ "ELSE",                   ELSE		},
+		{ "WHILE",                  WHILE		},
+		{ "FOR",                    FOR			},
+		{ "DO",                     DO			},
+		{ "GOTO",                   GOTO		},
+		{ "BREAK",                  BREAK		},
+		{ "CONTINUE",               CONTINUE		},
+		{ "RETURN",                 RETURN		},
+		{ "SWITCH",                 SWITCH		},
+		{ "CASE",                   CASE		},
+		{ "DEFAULT",                DEFAULT		},
+		{ "SIZEOF",                 SIZEOF		},
+		{ "AUTO",                   AUTO		},
+		{ "REGISTER",               REGISTER		},
+		{ "STATIC",                 STATIC		},
+		{ "EXTERN",                 EXTERN		},
+		{ "TYPEDEF",                TYPEDEF		},
+		{ "VOID",                   VOID		},
+		{ "CHAR",                   CHAR		},
+		{ "SHORT",                  SHORT		},
+		{ "INT",                    INT			},
+		{ "LONG",                   LONG		},
+		{ "FLOAT",                  FLOAT		},
+		{ "DOUBLE",                 DOUBLE		},
+		{ "SIGNED",                 SIGNED		},
+		{ "UNSIGNED",               UNSIGNED		},
+		{ "CONST",                  CONST		},
+		{ "VOLATILE",               VOLATILE		},
+		{ "STRUCT",                 STRUCT		},
+		{ "UNION",                  UNION		},
+		{ "ENUM",                   ENUM		},
+		{ "AND",                    AND			},
+		{ "TILDE",                  TILDE		},
+		{ "NOT",                    NOT			},
+		{ "LESSTHAN",               LESSTHAN		},
+		{ "GREATERTHAN",            GREATERTHAN		},
+		{ "XOR",                    XOR			},
+		{ "OR",                     OR			},
+		{ "PLUS",                   PLUS		},
+		{ "MINUS",                  MINUS		},
+		{ "SLASH",                  SLASH		},
+		{ "PERCENT",                PERCENT		},
+		{ "STAR",                   STAR		},
+		{ "DOT",                    DOT			},
+		{ "COLON",                  COLON		},
+		{ "QUERY",                  QUERY		},
+		{ "SEMI",                   SEMI		},
+		{ "COMMA",                  COMMA		},
+		{ "LPAREN",                 LPAREN		},
+		{ "RPAREN",                 RPAREN		},
+		{ "LBRACE",                 LBRACE		},
+		{ "RBRACE",                 RBRACE		},
+		{ "LBRAC",                  LBRAC		},
+		{ "RBRAC",                  RBRAC		},
+		{ "EQUALS",                 EQUALS		},
+		{ "STAR_EQUALS",            STAR_EQUALS		},
+		{ "SLASH_EQUALS",           SLASH_EQUALS	},
+		{ "PERCENT_EQUALS",         PERCENT_EQUALS	},
+		{ "PLUS_EQUALS",            PLUS_EQUALS		},
+		{ "MINUS_EQUALS",           MINUS_EQUALS	},
+		{ "LSHIFT_EQUALS",          LSHIFT_EQUALS	},
+		{ "RSHIFT_EQUALS",          RSHIFT_EQUALS	},
+		{ "AND_EQUALS",             AND_EQUALS		},
+		{ "XOR_EQUALS",             XOR_EQUALS		},
+		{ "OR_EQUALS",              OR_EQUALS		},
+		{ "ANDAND",                 ANDAND		},
+		{ "OROR",                   OROR		},
+		{ "EQEQ",                   EQEQ		},
+		{ "NOTEQ",                  NOTEQ		},
+		{ "GTEQ",                   GTEQ		},
+		{ "LESSEQ",                 LESSEQ		},
+		{ "LSHIFT",                 LSHIFT		},
+		{ "RSHIFT",                 RSHIFT		},
+		{ "PLUSPLUS",               PLUSPLUS		},
+		{ "MINUSMINUS",             MINUSMINUS		},
+		{ "ARROW",                  ARROW		},
+		{ "ELLIPSIS",               ELLIPSIS		},
+		{ "STRING_CONSTANT",        STRING_CONSTANT	},
+		{ "INTEGER_CONSTANT",       INTEGER_CONSTANT	},
+		{ "CHARACTER_CONSTANT",     CHARACTER_CONSTANT	},
+		{ "FLOATING_CONSTANT",      FLOATING_CONSTANT	},
+		{ "IDENTIFIER",             IDENTIFIER		},
+		{ "TYPEDEF_NAME",           TYPEDEF_NAME	},
+		{ "BADTOK",                 BADTOK		},
+		{ "EOF",                    0			},
 	};
 	static char buf[100];
 	int i;
