@@ -141,7 +141,7 @@ typedef union
 # define BADTOK 341
 
 #include <memory.h>
-#include <values.h>
+#include <limits.h>
 
 #ifdef __cplusplus
 
