@@ -713,6 +713,36 @@ type_t *type;
 	return buf;
 }
 
+/*
+**  Because of the recursive nature of the parser, the name of a
+**  base class is not always set up when the definition of a
+**  subclass is parsed.  This routine sets up the va_name field
+**  for a base class to the class name, if it is known, or to a
+**  safe fixed address otherwise.
+**
+**  We call this when the subclass definition is parsed and again
+**  when we need it for display.
+*/
+void
+ci_make_baseclass_name( v)
+var_t* v;
+{
+    static const char* unset_name = "Unknown BaseClass";
+
+    if (v->va_name == NULL || v->va_name == unset_name)
+    {
+	type_t* type = v->va_type;
+	if ( type->ty_code == DT_PTR_TO )
+	    type = type->ty_base;
+	if (type->ty_typedef != NULL &&  type->ty_typedef->td_name != NULL)
+	    v->va_name = type->ty_typedef->td_name;
+	else
+	    v->va_name = type->ty_aggr_or_enum->ae_tag;
+	if (  v->va_name == NULL)
+	     v->va_name = unset_name;
+    }
+}
+
 /*  Rewrite any parameters of type "array of T" to "pointer to T"
  */
 void
