@@ -7,8 +7,11 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-/* @(#)ci_types.h	1.3 4/7/91 (UKC) */
+/* @(#)ci_types.h	1.4 18/10/92 (UKC) */
 
+declaration_t *ci_make_declaration PROTO((class_t class, type_t *basetype,
+					  qualifiers_t qualifiers,
+					  declarator_t *declarators));
 void report_redecl PROTO((const char *name,
 				 nametype_t nametype, lexinfo_t *this_lx,
 				 nametype_t prev_nametype, lexinfo_t *prev_lx));
@@ -24,9 +27,11 @@ void ci_show_type PROTO((type_t *type, const char *what));
 taddr_t ci_align_addr PROTO((taddr_t addr, int alignment));
 taddr_t ci_align_addr_for_type PROTO((taddr_t addr, type_t *type));
 int ci_type_alignment PROTO((type_t *type));
+optype_t ci_typecode_to_cvt_op PROTO((typecode_t typecode));
+void ci_rewrite_func_params PROTO((funcret_t *fr));
 
 void ci_add_type_specifier PROTO((declaration_t *declaration, type_t *type));
-void ci_fix_signed_and_unsigned PROTO((declaration_t *declaration));
+type_t *ci_get_dn_basetype PROTO((declaration_t *declaration));
 
 #define IS_ARITHMETIC_TYPE(code) \
 	     (ci_is_integral(code) || ((code) == TY_FLOAT) || ((code) == TY_DOUBLE))
