@@ -37,6 +37,8 @@
 #define MR_VAR_COLLAPSE_COMPLETELY	'C'
 #define MR_VAR_WANT_TYPEDEFS		'U'
 #define MR_VAR_NO_TYPEDEFS		'I'
+#define MR_VAR_SHOW_HIERARCHY		'y' /* RCB */
+#define MR_VAR_SHOW_STATICS		'S' /* RCB */
 #define MR_VAR_RWCSTRING		'b'
 #define MR_VAR_ACCELERATOR_ACTION	'q' /* RCB */
 
@@ -182,6 +184,7 @@
 #define MR_HELP_3D			'L'
 #define MR_HELP_3E			'M'
 #define MR_HELP_3F			'N'
+#define MR_HELP_3G			'n'
 #define MR_HELP_4			'4'
 #define MR_HELP_5A			'S'
 #define MR_HELP_5B			'T'
