@@ -4,7 +4,7 @@
 static MENU MM1 = {
 	0x40,
 	0,
-	"expand",
+	"Expand",
 	115,
 	-1,
 	-1,
@@ -13,6 +13,8 @@ static MENU MM1 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	NULL,
 	NULL,
@@ -22,7 +24,7 @@ static MENU MM1 = {
 static MENU MM2 = {
 	0x40,
 	0,
-	"collapse",
+	"Collapse",
 	104,
 	125,
 	-1,
@@ -31,6 +33,8 @@ static MENU MM2 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	NULL,
 	NULL,
@@ -49,6 +53,8 @@ static MENU MM3 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	&MM1,
 	&MM2,
@@ -58,7 +64,7 @@ static MENU MM3 = {
 static MENU MM4 = {
 	0x40,
 	0,
-	"add entry",
+	"Add entry",
 	97,
 	252,
 	-1,
@@ -67,6 +73,8 @@ static MENU MM4 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	NULL,
 	NULL,
@@ -76,7 +84,7 @@ static MENU MM4 = {
 static MENU MM5 = {
 	0x40,
 	0,
-	"reset env",
+	"Reset env",
 	114,
 	379,
 	-1,
@@ -85,6 +93,8 @@ static MENU MM5 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	NULL,
 	NULL,
@@ -103,6 +113,8 @@ static MENU MM6 = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	&MM4,
 	&MM5,
@@ -121,6 +133,8 @@ MENU envhead_men = {
 	0,
 	0,
 	0,
+	0,
+	0,
 	NULL,
 	&MM3,
 	&MM6,
