@@ -1900,9 +1900,9 @@ restore_cached_bpts()
 {
   bool junk;
 
-  read_config_file("/tmp/temp-ups.bpts", TRUE, TRUE, TRUE, FALSE, &junk);
-/*  if (unlink("/tmp/temp-ups.bpts") != 0)*/
-/*    errf("Can't unlink /tmp/temp-ups.bpts");*/
+  read_config_file(get_temp_bpt_filename(), TRUE, TRUE, TRUE, FALSE, &junk);
+  /*  if (unlink(get_temp_bpt_filename()) != 0)*/
+  /*    errf("Can't unlink `%s'", get_temp_bpt_filename());*/
 }
 
 /*  Add the breakpoint header to the display.  Called on startup from main().
