@@ -298,6 +298,9 @@ typedef struct var_s {
 #define VA_EXT_RELOC		0x0008	/* use external relocation (ci) */
 #define VA_TODO			0x0010	/* tmp marker flag (ci_compile.c) */
 #define VA_ON_VLIST		0x0020	/* on tx_vlist (ci_compile.c) */
+#define VA_BASECLASS		0x0040	/* Is base class for a g++ class (RCB) */
+#define VA_STATIC_MEMBER	0x0080	/* Static member of a g++ class (RCB) */
+#define VA_VTBLPTR		0x0100	/* Is virtual table pointer (RCB) */
 
 /*  Element in the list of functions attached to a source file.
  */
