@@ -23,8 +23,8 @@ char ups_ui_layout_c_sccsid[] = "@(#)ui_layout.c	1.32 20 Jun 1995 (UKC)";
 
 #include <local/ukcprog.h>
 #include <local/obj/obj.h>
-#include <local/obj/o_mkobj.h>
 #include <local/obj/o_priv.h>
+#include <local/obj/o_mkobj.h>
 #include <local/obj/o_sel.h>
 #include <local/obj/mhole.h>
 #include <local/obj/newtb.h>
@@ -728,6 +728,7 @@ add_outwin()
 {
 	Region *srcwin_region, *root;
 	double outwin_fraction;
+	int oldstate;
 	
 	srcwin_region = get_current_srcwin_region();
 
@@ -752,7 +753,9 @@ add_outwin()
 	outwin_fraction = (double)get_default("OutwinPercent", 10, 1) / 100;
 	create_outwin(root, outwin_fraction);
 
+	oldstate = td_set_obj_updating(OBJ_UPDATING_ON);
 	re_redraw_root(EV_WINDOW_RESIZED, TRUE);
+	td_set_obj_updating(oldstate);
 }
 
 Outwin *
@@ -763,8 +766,8 @@ get_or_create_outwin()
 	ow = get_current_outwin();
 
 	if (ow == NULL) {
-		add_outwin();
-		ow = get_current_outwin();
+	  add_outwin();
+	  ow = get_current_outwin();
 	}
 
 	return ow;
@@ -1309,7 +1312,7 @@ display_area_input(region, ev)
 	      do_block((objid_t)sel->se_code, !obj->ob_child ?  
 		       MR_BLOCK_EXPAND : MR_BLOCK_COLLAPSE, (char *)NULL);
 	    else
-	      do_block((objid_t)sel->se_code, MR_DISPLAY_SOURCE, (char *)NULL);
+	      do_block((objid_t)sel->se_code, MR_BLOCK_DISPLAY_SOURCE, (char *)NULL);
 	    break;
 	  }
 	  sel = sel->se_next;
@@ -1335,7 +1338,17 @@ Region *region;
 char *data;
 bool force;
 {
-	return field_edit_finish(data, force);
+  bool result;
+  cursor_t old_cursor = wn_get_window_cursor(WN_STDWIN);
+  set_bm_cursor(WN_STDWIN, CU_WAIT);
+
+  indicate_target_menu_search(1);
+  target_menu_search_disabled(1, 0); /* set */
+  result = field_edit_finish(data, force);
+  target_menu_search_disabled(0, 1); /* reset */
+  indicate_target_menu_search(0);
+  wn_define_cursor(WN_STDWIN, old_cursor);
+  return result;
 }
 
 static void
@@ -1472,10 +1485,16 @@ int cmd;
 		break;
 	case MR_OUTWIN_SEARCH_FORWARDS:
 	case MR_OUTWIN_SEARCH_BACKWARDS:
+	   {
+		int offset = 0;
 		pattern = get_typing_line_string();
-		outwin_search(ow, pattern, cmd == MR_OUTWIN_SEARCH_FORWARDS);
+		/* Skip initial direction indicator if any */
+		if ( *pattern == '/' || *pattern == '?') 
+		   offset=1;
+		outwin_search(ow, pattern+offset, cmd == MR_OUTWIN_SEARCH_FORWARDS);
 		free(pattern);
 		break;
+	   }
 	default:
 		panic("bad cmd in owmf");
 	}
@@ -1708,12 +1727,6 @@ re_draw_reason_t draw_reason;
 		draw_mousehole(wn, 0, 0, width, height);
 	}
 }
-bool
-target_process_running()
-{
-  extern MENU tgt_stop_men;
-  return Mselectable(&tgt_stop_men);
-}
 
 /*  For setting and getting redirection to outwin
  */
