@@ -477,13 +477,16 @@ int md, cmd;
 		break;
 	case MR_SRCWIN_SEARCH_FORWARDS:
 	case MR_SRCWIN_SEARCH_BACKWARDS:
+	   {
+		int offset = 0;
 		pattern = get_typing_line_string();
-		/* Skip initial direction indicator in any */
+		/* Skip initial direction indicator if any */
 		if ( *pattern == '/' || *pattern == '?') 
-		   pattern++;
-		srcwin_search(sw, pattern, cmd == MR_SRCWIN_SEARCH_FORWARDS);
+		   offset=1;
+		srcwin_search(sw, pattern+offset, cmd == MR_SRCWIN_SEARCH_FORWARDS);
 		free(pattern);
 		break;
+	   }
 	case MR_SRCWIN_UP_STACK:
 		change_displayed_stack_level(TRUE);
 		break;
@@ -583,7 +586,8 @@ int lnum;
 {
         /* RGA add ' ' so we can click on 'a -> b" */
 	/* RCB: add '#' to check for "#stop" */
-	static const char pat[] = "!-a-zA-Z0-9$_.>% #";
+	/* RCB: Add ':' for '::' */
+	static const char pat[] = "!-a-zA-Z0-9$_.>% #:";
 	Edit_display *display;
 	Edit_buffer *buffer;
 	size_t start, lim;
@@ -684,7 +688,7 @@ int lnum;
 	    free(text);
 	  }
 	}
-      }
+}
 
 static bool
 check_repltab(ap, path, p_rlist, p_mtime)
@@ -993,13 +997,26 @@ event_t *ev;
 	buttons = ev->ev_buttons & (B_ANY | B_SHIFT_MASK | B_CONTROL_MASK | B_MULT_CLICKS);
 
 	if (buttons == B_LEFT &&  srcinfo.have_point) {
+            cursor_t old_cursor = wn_get_window_cursor(WN_STDWIN);
+	    set_bm_cursor(WN_STDWIN, CU_WAIT);
 	    srcwin_have_editblock(sw, region, ev);
+	    indicate_target_menu_search(1);
+	    target_menu_search_disabled(1, 0); /* set */
 	    do_var_selection(sw, ev, srcinfo.point,
 			     srcinfo.fil, srcinfo.fil_lnum);
+	    target_menu_search_disabled(0, 1); /* reset */
+	    indicate_target_menu_search(0);
+	    wn_define_cursor(WN_STDWIN, old_cursor);
 	}
 	if (buttons == (B_LEFT | B_SHIFT_MASK) && srcinfo.have_point) {
-		do_expr_selection(sw, ev, srcinfo.point,
-				 srcinfo.fil, srcinfo.fil_lnum);
+          cursor_t old_cursor = wn_get_window_cursor(WN_STDWIN);
+	  indicate_target_menu_search(1);
+	  target_menu_search_disabled(1, 0); /* set */
+	  do_expr_selection(sw, ev, srcinfo.point,
+			    srcinfo.fil, srcinfo.fil_lnum);
+	  target_menu_search_disabled(0, 1); /* reset */
+	  indicate_target_menu_search(0);
+	  wn_define_cursor(WN_STDWIN, old_cursor);
 	}
 	/* RCB: Check for Double click on breakpoint */
 	if ((buttons & B_LEFT) && (buttons&B_MULT_CLICKS) &&  srcinfo.have_point) {
