@@ -42,6 +42,12 @@ char ups_va_edit_c_sccsid[] = "@(#)va_edit.c	1.31 04 Jun 1995 (UKC)";
 double strtod();
 #endif
 
+#if (defined(__GNUC__) && !defined (OS_SUNOS_4)) || defined(HAVE_LONG_LONG)
+typedef unsigned long long	largeint;
+#else
+typedef unsigned long		largeint;
+#endif
+
 static void var_draw PROTO((draw_t *p_dets));
 static bool var_quitfunc PROTO((objid_t obj, char *new_text,
 				const char **p_display_string));
@@ -54,7 +60,7 @@ static bool update_ilist PROTO((dvar_t *dv, const char *decl));
 static int get_value PROTO((bool words_big_endian, type_t *type,
 			    const char *vstr, value_t *vl,
 			    vformat_t *p_vformat));
-static int get_integer PROTO((const char *vstr, unsigned long *p_val,
+static int get_integer PROTO((const char *vstr, largeint *p_val,
 			      vformat_t *p_vformat, bool boolean));
 static enum_member_t *find_enum_member PROTO((aggr_or_enum_def_t *ae, const char *name));
 static int get_float PROTO((const char *vstr, double *p_dval));
@@ -637,10 +643,10 @@ long *p_fg, *p_bg;
 	  if (colors[0].co_pixel == -1) {
 		const char *colorname;
 
-		if ((colorname = wn_get_default("HighlightColor")) != NULL)
-			wn_parse_color(colorname, &colors[1]);
-		else
-		  colorname = "red";
+		colorname = wn_get_default("HighlightColor");
+		if ( colorname == NULL)
+		    colorname = "red";
+		wn_parse_color(colorname, &colors[1]);
 
 		colors[0].co_pixel = WN_BG;
 
@@ -716,11 +722,12 @@ long *p_bg, *p_fg;
 	  if (colors[0].co_pixel == -1) {
 		const char *colorname;
 
-		if ((colorname = wn_get_default("VariableColor")) != NULL)
-			wn_parse_color(colorname, &colors[1]);
-		else
+		colorname = wn_get_default("VariableColor");
+		if ( colorname == NULL)
 		  colorname = "purple";
 
+		wn_parse_color(colorname, &colors[1]);
+
 		colors[0].co_pixel = WN_BG;
 
 		if (!wn_use_alloc_color(0))
@@ -1108,7 +1115,7 @@ const char *vstr;
 value_t *vl;
 vformat_t *p_vformat;
 {
-	unsigned long val;
+	largeint val;
 	double d;
 	int res;
 	int boolean = (type->ty_code==TY_BOOLEAN);
@@ -1155,6 +1162,15 @@ vformat_t *p_vformat;
 			res = -1;
 		break;
 
+#if (defined(__GNUC__) && !defined (OS_SUNOS_4)) || defined(HAVE_LONG_LONG)
+	case TY_LONGLONG:
+	case TY_ULONGLONG:
+		if (get_integer(vstr, &val, p_vformat, boolean) == 0)
+			vl->vl_longlong = val;
+		else
+			res = -1;
+		break;
+#endif
 
 	case TY_INT:
 	case TY_UINT:
@@ -1350,13 +1366,13 @@ int *p_ints;
 static int
 get_integer(vstr, p_val, p_vformat, boolean)
 const char *vstr;
-unsigned long *p_val;
+largeint *p_val;
 vformat_t *p_vformat;
 bool boolean;
 {
 	const char *s, *endpos;
 	int radix;
-	unsigned long val;
+	largeint val;
 	vformat_t vformat;
 	bool negative;
 
@@ -1541,6 +1557,9 @@ draw_t fdets;
 	case DT_PTR_TO:
 	case TY_FLOAT:
 	case TY_DOUBLE:
+	case TY_LONGLONG:
+	case TY_ULONGLONG:
+	/* case TY_LONGDOUBLE: */
 	case TY_BOOLEAN:
 		can_edit = TRUE;
 		break;
