@@ -576,9 +576,14 @@ char *arg;
 			var_t *v;
 			v = de->de_type->ty_base->ty_aggr_or_enum->ae_aggr_members;
 			for (; v != NULL; v = v->va_next) {
-				if (find_var((objid_t)de, v) == NULL)
-					add_var_object(obj, v, OBJ_FIRST_CHILD,
-						       FALSE);
+			    /*
+			    **   Skip static members, which we display as globals.
+			    **   Skip vector tables, if thats what the user wants to see
+			    **   the user should use the expand_hierarchy command.
+			    */
+			    if (   !( v->va_flags & (VA_STATIC_MEMBER|VA_VTBLPTR))
+			        && find_var((objid_t)de, v) == NULL)
+				    add_var_object(obj, v, OBJ_FIRST_CHILD, FALSE);
 			}
 			break;
 		    }
@@ -593,6 +598,12 @@ char *arg;
 			break;
 		}
 
+		break;
+	case MR_VAR_SHOW_HIERARCHY:
+		expand_hierarchy(obj,de->de_type);
+		break;
+	case MR_VAR_SHOW_STATICS:
+	        expand_statics(de->de_type);
 		break;
 	case MR_VAR_DUP:
 		add_to_new_selection((objid_t)dup_dexpr(de));
