@@ -32,6 +32,7 @@ char ups_va_menu_c_sccsid[] = "@(#)va_menu.c	1.34 04 Jun 1995 (UKC)";
 #include "ui.h"
 #include "obj_stack.h"
 #include "tdr.h"
+#include "expr.h" /* RCB, for show_static_member() */
 
 static taddr_t get_dv_addr PROTO((objid_t par, var_t *v, ilist_t *ilist,
 				  bool *p_decl_may_have_changed));
@@ -325,7 +326,7 @@ const char** p_format;
 const char** start;
 {
     const char* format = *p_format;
-    char* end;
+    const char* end;
     char C;
 
     /*   Skip white space and comments */
@@ -695,7 +696,7 @@ bool restoring;
 
 	    if (   ISDERIV(typecode)
 		|| DF_NONE == (dv->dv_format = check_for_special_format
-			       ((char *)v->va_name, typecode )))
+			       (v->va_name, typecode )))
 	    dv->dv_format = default_format(v->va_type, type );
 	}
 
@@ -974,6 +975,98 @@ display_mangled_component(name, component, obj)
   return found;
 }
 
+void
+expand_statics(type)
+type_t* type;
+{
+    var_t* v;
+    const char* mesg = NULL;
+    bool found  = FALSE;
+    switch(type->ty_code) {
+    case TY_STRUCT:
+    case TY_UNION:
+	break;
+    case TY_U_STRUCT:
+	mesg = "Can't expand statics for  undefined structure";
+	break;
+    case TY_U_UNION:
+	mesg = "Can't expand statics for  undefined union";
+	break;
+    default:
+	mesg = "Not a structure or union";
+	break;
+    }
+    if (mesg != NULL) {
+        errf(mesg);
+        return;
+    }
+
+    for (v = type->ty_aggr_or_enum->ae_aggr_members;
+		     v != NULL;
+		     v = v->va_next)
+    {
+	if ( v->va_flags & VA_STATIC_MEMBER)
+	{
+	    found = TRUE;
+	    show_static_member(type,v);
+	}
+    }
+    if ( !found)
+	errf("No static members");
+}
+taddr_t
+expand_hierarchy(obj, type)
+objid_t obj;
+type_t* type;
+{
+    dvar_t* dv_vtbl = NULL;
+    taddr_t vtbl_addr;
+
+    var_t *v;
+
+    while ( type->ty_code == DT_PTR_TO)
+	type = type->ty_base;
+
+    if ( type->ty_code != TY_STRUCT)
+	return BAD_ADDR;
+    for (v = type->ty_aggr_or_enum->ae_aggr_members;
+	v != NULL;
+	v = v->va_next)
+    {
+	if ( v->va_flags & VA_VTBLPTR)
+	{
+	    dv_vtbl = ( dvar_t*) find_var(obj, v);
+	    if ( dv_vtbl == NULL)
+	        dv_vtbl = (dvar_t*) add_var_object(obj, v, OBJ_FIRST_CHILD, FALSE);
+	    if ( dv_vtbl->dv_addr != BAD_ADDR)
+	    {
+		target_t* xp = get_current_target();
+		if (dread(xp, dv_vtbl->dv_addr, (char *)&vtbl_addr, sizeof(vtbl_addr)) != 0)
+		    vtbl_addr = BAD_ADDR;
+	    } else
+		    vtbl_addr = BAD_ADDR;
+	} else if ( v->va_flags & VA_BASECLASS)
+	{
+	    taddr_t sub_addr;
+	    dvar_t* dv = ( dvar_t*) find_var(obj, v);
+	    if ( dv == NULL)
+	        dv = (dvar_t*) add_var_object(obj, v, OBJ_FIRST_CHILD, FALSE);
+	    sub_addr = expand_hierarchy((objid_t)dv,deref_aggr(dv));
+
+
+	    /* If the base class displayed the same vtbl as previously displayed on
+	    ** this level, remove the initial display
+	    */
+	    if ( dv_vtbl && ( sub_addr == vtbl_addr) )
+	    {
+		remove_object((objid_t)dv_vtbl, OBJ_SELF);
+		dv_vtbl = NULL;
+	    }
+	}
+    }
+    return vtbl_addr;
+}
+
 /*  Process the return from a var menu.
  */
 static void
@@ -1086,9 +1179,8 @@ char *arg;
 
 		if (   ISDERIV(type->ty_code)
 		    ||    DF_NONE
-		       == (dv->dv_format = check_for_special_format((char *) v->va_name, type->ty_code )))
+		       == (dv->dv_format = check_for_special_format( v->va_name, type->ty_code )))
 
-/* AMC Change: changed indentation */
 		    dv->dv_format = default_format(v->va_type, type);
 
 		oldstate = td_set_obj_updating(OBJ_UPDATING_OFF);
@@ -1159,9 +1251,23 @@ char *arg;
 		for (v = type->ty_aggr_or_enum->ae_aggr_members;
 		     v != NULL;
 		     v = v->va_next)
-			if (find_var((objid_t)dv, v) == NULL)
+		{
+		    /*
+		    **   Skip static members, which we display as globals.
+		    **   Skip vector tables, if thats what the user wants to see
+		    **   the user should use the expand_hierarchy command.
+		    */
+		    if (   !( v->va_flags & (VA_STATIC_MEMBER|VA_VTBLPTR))
+			&& find_var((objid_t)dv, v) == NULL)
 				add_var_object((objid_t)dv, v, OBJ_FIRST_CHILD,
 					       FALSE);
+		}
+		break;
+	case MR_VAR_SHOW_HIERARCHY:
+	        expand_hierarchy((objid_t)dv,deref_aggr(dv));
+		break;
+	case MR_VAR_SHOW_STATICS:
+	        expand_statics(deref_aggr(dv));
 		break;
 	case MR_VAR_DUP:
 		dv2 = new_dvar();
@@ -1372,6 +1478,12 @@ int **obj_var;
       str_match = !strcmp(v->va_name, name);     
     else
       str_match = !!strstr(v->va_name, name);
+    if ( str_match && v->va_flags & VA_STATIC_MEMBER)
+    {
+	show_static_member(type,v);
+	found = 1;
+	break;
+    } else
     /* RGA Note return value of get_member_of_aggr() changed from 2.45.2 */
     if (str_match && !get_member_of_aggr(obj, v->va_name, &sub_v, &addr, &btype))
     {
