@@ -87,6 +87,7 @@ typedef struct dvarst {
 
 #define DVF_VAL_CHANGED		0x001	/* value changed since last time */
 #define DVF_NO_TYPEDEFS		0x002	/* don't use typedefs in decls */
+#define DVF_BAD_FORMAT_HINT	0x004	/* bad hint - restore old */
 
 /*  Field numbers in a variable object.
  */
@@ -96,7 +97,7 @@ typedef struct dvarst {
 
 /*  'long long' support ?
  */
-#if (defined(__GNUC__) && !defined (OS_SUNOS_4)) || defined(HAVE_LONG_LONG)
+#if defined(HAVE_LONG_LONG)
 typedef long long vlong;
 #else
 typedef long vlong;
@@ -180,6 +181,8 @@ bool maybe_restore_var_state PROTO((objid_t par, dvar_t *dv));
  */
 #ifdef OBJ_H_INCLUDED
 void note_var_change PROTO((objid_t obj));
+taddr_t expand_hierarchy PROTO((objid_t obj, type_t* type));
+void expand_statics PROTO(( type_t* type));
 #endif
 void note_format_change PROTO((dvar_t *dv));
 bool get_num PROTO((char **p_line, const char *what, int *p_val));
