@@ -15,6 +15,7 @@ char ups_va_val_c_sccsid[] = "@(#)va_val.c	1.39 04 Jun 1995 (UKC)";
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <local/ukcprog.h>
 #include "ups.h"
@@ -34,6 +35,8 @@ static int get_fieldval PROTO((bool bits_big_endian, int word,
 static int is_signed_format PROTO((vformat_t format));
 static void note_badaddr PROTO((char *buf, size_t bufsize, language_t language, 
                                 taddr_t addr));
+static void vtbl_to_string PROTO(( char *buf, size_t bufsize, taddr_t addr,
+				type_t *type, vformat_t format));
 
 /*  Read and display a C NUL terminated string.
  *
@@ -199,6 +202,45 @@ vformat_t format;
 		int_to_string(buf, bufsize, (vlong)addr, format);
 	}
 }
+static void
+vtbl_to_string(buf, bufsize, addr, type, format)
+char *buf;
+size_t bufsize;
+taddr_t addr;
+type_t *type;
+vformat_t format;
+{
+    size_t offset;
+    const char* name = lookup_global_name(addr, &offset);
+    if ( name )
+    {
+	char* dmangle = demangle_name_3(name);
+	if ( dmangle)
+	{
+	    strncpy(buf, dmangle, bufsize-1);
+	    free(dmangle);
+	}
+	else
+	    strncpy(buf, name, bufsize-1);
+	buf[bufsize-1] = 0;
+
+	/* If address was not an exact match, add in the offset
+	**   ( This happens for SC4 when multiple inheritance is used )
+	*/
+	if ( offset)
+	{
+	   int len = strlen(buf);
+	   int maxcp = bufsize-1-len;
+	   if ( maxcp > 0 )
+	   {
+	       char offset_buff[12];
+	       sprintf(offset_buff,"+0x%lx",offset);
+	       strncpy(buf+len, offset_buff, maxcp);
+	   }
+	}
+    } else
+	addr_to_string(buf, bufsize, addr, type, format);
+}
 
 /*  Construct a value string for an integer type variable using the format.
  */
@@ -385,6 +427,9 @@ dvar_t *dv;
 	v = dv->dv_var;
 	addr = dvar_addr(dv);
         
+	if (dv->dv_flags & DVF_BAD_FORMAT_HINT)
+		return "<Bad format hint>";
+
 	if (addr == BAD_ADDR)
 		return "<Address out of range>";
 
@@ -401,7 +446,12 @@ dvar_t *dv;
 			note_badaddr(buf, sizeof(buf), v->va_language, addr);
 			return buf;
 		}
-		addr_to_string(buf, sizeof(buf), vl.vl_addr,
+		if ( v->va_flags & VA_VTBLPTR )
+		    vtbl_to_string(buf, sizeof(buf), vl.vl_addr,
+							type, dv->dv_format);
+
+		else
+		    addr_to_string(buf, sizeof(buf), vl.vl_addr,
 							type, dv->dv_format);
 		return buf;
 	}
@@ -448,11 +498,29 @@ dvar_t *dv;
 		break;
 
 	case TY_BOOLEAN:
-		if (DREAD_VAR(xp, addr, vl.vl_logical)) {
-				strcpy(buf, vl.vl_logical ? "true"
-				       			  : "false");
+	     {
+		int val;
+		int ret = 0;
+		switch ( type->ty_size)
+		{
+		case 1:
+		    ret = DREAD_VAR(xp, addr, vl.vl_char);
+		    val = vl.vl_char;
+		    break;
+		case 2:
+		    ret = DREAD_VAR(xp, addr, vl.vl_short);
+		    val = vl.vl_short;
+		    break;
+		case 4:
+		    ret = DREAD_VAR(xp, addr, vl.vl_long);
+		    val = vl.vl_long;
+		    break;
 		}
+
+		if (ret)
+		   strcpy(buf, val ? "true" : "false");
 		break;
+	     }
 
 	case TY_LOGICAL:
 		if (DREAD_VAR(xp, addr, vl.vl_logical)) {
@@ -700,4 +768,3 @@ taddr_t addr;
 		strnf(buf, bufsize, "<bad address 0x%lx>", addr);
 	}
 }
-	
